"""
Python module generated from Java source file com.google.common.math.Stats

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import MoreObjects
from com.google.common.base import Objects
from com.google.common.math import *
from java.io import Serializable
from java.util import Iterator
from java.util.stream import Collector
from java.util.stream import DoubleStream
from java.util.stream import IntStream
from java.util.stream import LongStream
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Stats(Serializable):
    """
    A bundle of statistical summary values -- sum, count, mean/average, min and max, and several
    forms of variance -- that were computed from a single set of zero or more floating-point values.
    
    There are two ways to obtain a `Stats` instance:
    
    
      - If all the values you want to summarize are already known, use the appropriate `Stats.of` factory method below. Primitive arrays, iterables and iterators of any kind of
          `Number`, and primitive varargs are supported.
      - Or, to avoid storing up all the data first, create a StatsAccumulator instance,
          feed values to it as you get them, then call StatsAccumulator.snapshot.
    
    
    Static convenience methods called `meanOf` are also provided for users who wish to
    calculate *only* the mean.
    
    **Java 8+ users:** If you are not using any of the variance statistics, you may wish to use
    built-in JDK libraries instead of this class.

    Author(s)
    - Kevin Bourrillion

    Since
    - 20.0
    """

    @staticmethod
    def of(values: Iterable["Number"]) -> "Stats":
        """
        Returns statistics over a dataset containing the given values.

        Arguments
        - values: a series of values, which will be converted to `double` values (this may
            cause loss of precision)
        """
        ...


    @staticmethod
    def of(values: Iterator["Number"]) -> "Stats":
        """
        Returns statistics over a dataset containing the given values. The iterator will be completely
        consumed by this method.

        Arguments
        - values: a series of values, which will be converted to `double` values (this may
            cause loss of precision)
        """
        ...


    @staticmethod
    def of(*values: Tuple[float, ...]) -> "Stats":
        """
        Returns statistics over a dataset containing the given values.

        Arguments
        - values: a series of values
        """
        ...


    @staticmethod
    def of(*values: Tuple[int, ...]) -> "Stats":
        """
        Returns statistics over a dataset containing the given values.

        Arguments
        - values: a series of values
        """
        ...


    @staticmethod
    def of(*values: Tuple[int, ...]) -> "Stats":
        """
        Returns statistics over a dataset containing the given values.

        Arguments
        - values: a series of values, which will be converted to `double` values (this may
            cause loss of precision for longs of magnitude over 2^53 (slightly over 9e15))
        """
        ...


    @staticmethod
    def of(values: "DoubleStream") -> "Stats":
        """
        Returns statistics over a dataset containing the given values. The stream will be completely
        consumed by this method.
        
        If you have a `Stream<Double>` rather than a `DoubleStream`, you should collect
        the values using .toStats() instead.

        Arguments
        - values: a series of values

        Since
        - 28.2
        """
        ...


    @staticmethod
    def of(values: "IntStream") -> "Stats":
        """
        Returns statistics over a dataset containing the given values. The stream will be completely
        consumed by this method.
        
        If you have a `Stream<Integer>` rather than an `IntStream`, you should collect
        the values using .toStats() instead.

        Arguments
        - values: a series of values

        Since
        - 28.2
        """
        ...


    @staticmethod
    def of(values: "LongStream") -> "Stats":
        """
        Returns statistics over a dataset containing the given values. The stream will be completely
        consumed by this method.
        
        If you have a `Stream<Long>` rather than a `LongStream`, you should collect the
        values using .toStats() instead.

        Arguments
        - values: a series of values, which will be converted to `double` values (this may
            cause loss of precision for longs of magnitude over 2^53 (slightly over 9e15))

        Since
        - 28.2
        """
        ...


    @staticmethod
    def toStats() -> "Collector"["Number", "StatsAccumulator", "Stats"]:
        """
        Returns a Collector which accumulates statistics from a java.util.stream.Stream
        of any type of boxed Number into a Stats. Use by calling `boxedNumericStream.collect(toStats())`. The numbers will be converted to `double` values
        (which may cause loss of precision).
        
        If you have any of the primitive streams `DoubleStream`, `IntStream`, or `LongStream`, you should use the factory method .of instead.

        Since
        - 28.2
        """
        ...


    def count(self) -> int:
        """
        Returns the number of values.
        """
        ...


    def mean(self) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of the
        values. The count must be non-zero.
        
        If these values are a sample drawn from a population, this is also an unbiased estimator of
        the arithmetic mean of the population.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains Double.NaN then the result is Double.NaN. If it
        contains both Double.POSITIVE_INFINITY and Double.NEGATIVE_INFINITY then the
        result is Double.NaN. If it contains Double.POSITIVE_INFINITY and finite values
        only or Double.POSITIVE_INFINITY only, the result is Double.POSITIVE_INFINITY.
        If it contains Double.NEGATIVE_INFINITY and finite values only or Double.NEGATIVE_INFINITY only, the result is Double.NEGATIVE_INFINITY.
        
        If you only want to calculate the mean, use .meanOf instead of creating a Stats instance.

        Raises
        - IllegalStateException: if the dataset is empty
        """
        ...


    def sum(self) -> float:
        """
        Returns the sum of the values.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains Double.NaN then the result is Double.NaN. If it
        contains both Double.POSITIVE_INFINITY and Double.NEGATIVE_INFINITY then the
        result is Double.NaN. If it contains Double.POSITIVE_INFINITY and finite values
        only or Double.POSITIVE_INFINITY only, the result is Double.POSITIVE_INFINITY.
        If it contains Double.NEGATIVE_INFINITY and finite values only or Double.NEGATIVE_INFINITY only, the result is Double.NEGATIVE_INFINITY.
        """
        ...


    def populationVariance(self) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Variance#Population_variance">population
        variance</a> of the values. The count must be non-zero.
        
        This is guaranteed to return zero if the dataset contains only exactly one finite value. It
        is not guaranteed to return zero when the dataset consists of the same value multiple times,
        due to numerical errors. However, it is guaranteed never to return a negative result.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains any non-finite values (Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, or Double.NaN) then the result is Double.NaN.

        Raises
        - IllegalStateException: if the dataset is empty
        """
        ...


    def populationStandardDeviation(self) -> float:
        """
        Returns the <a
        href="http://en.wikipedia.org/wiki/Standard_deviation#Definition_of_population_values">
        population standard deviation</a> of the values. The count must be non-zero.
        
        This is guaranteed to return zero if the dataset contains only exactly one finite value. It
        is not guaranteed to return zero when the dataset consists of the same value multiple times,
        due to numerical errors. However, it is guaranteed never to return a negative result.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains any non-finite values (Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, or Double.NaN) then the result is Double.NaN.

        Raises
        - IllegalStateException: if the dataset is empty
        """
        ...


    def sampleVariance(self) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Variance#Sample_variance">unbiased sample
        variance</a> of the values. If this dataset is a sample drawn from a population, this is an
        unbiased estimator of the population variance of the population. The count must be greater than
        one.
        
        This is not guaranteed to return zero when the dataset consists of the same value multiple
        times, due to numerical errors. However, it is guaranteed never to return a negative result.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains any non-finite values (Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, or Double.NaN) then the result is Double.NaN.

        Raises
        - IllegalStateException: if the dataset is empty or contains a single value
        """
        ...


    def sampleStandardDeviation(self) -> float:
        """
        Returns the <a
        href="http://en.wikipedia.org/wiki/Standard_deviation#Corrected_sample_standard_deviation">
        corrected sample standard deviation</a> of the values. If this dataset is a sample drawn from a
        population, this is an estimator of the population standard deviation of the population which
        is less biased than .populationStandardDeviation() (the unbiased estimator depends on
        the distribution). The count must be greater than one.
        
        This is not guaranteed to return zero when the dataset consists of the same value multiple
        times, due to numerical errors. However, it is guaranteed never to return a negative result.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains any non-finite values (Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, or Double.NaN) then the result is Double.NaN.

        Raises
        - IllegalStateException: if the dataset is empty or contains a single value
        """
        ...


    def min(self) -> float:
        """
        Returns the lowest value in the dataset. The count must be non-zero.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains Double.NaN then the result is Double.NaN. If it
        contains Double.NEGATIVE_INFINITY and not Double.NaN then the result is Double.NEGATIVE_INFINITY. If it contains Double.POSITIVE_INFINITY and finite values
        only then the result is the lowest finite value. If it contains Double.POSITIVE_INFINITY only then the result is Double.POSITIVE_INFINITY.

        Raises
        - IllegalStateException: if the dataset is empty
        """
        ...


    def max(self) -> float:
        """
        Returns the highest value in the dataset. The count must be non-zero.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains Double.NaN then the result is Double.NaN. If it
        contains Double.POSITIVE_INFINITY and not Double.NaN then the result is Double.POSITIVE_INFINITY. If it contains Double.NEGATIVE_INFINITY and finite values
        only then the result is the highest finite value. If it contains Double.NEGATIVE_INFINITY only then the result is Double.NEGATIVE_INFINITY.

        Raises
        - IllegalStateException: if the dataset is empty
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        
        
        **Note:** This tests exact equality of the calculated statistics, including the floating
        point values. Two instances are guaranteed to be considered equal if one is copied from the
        other using `second = new StatsAccumulator().addAll(first).snapshot()`, if both were
        obtained by calling `snapshot()` on the same StatsAccumulator without adding any
        values in between the two calls, or if one is obtained from the other after round-tripping
        through java serialization. However, floating point rounding errors mean that it may be False
        for some instances where the statistics are mathematically equal, including instances
        constructed from the same values in a different order... or (in the general case) even in the
        same order. (It is guaranteed to return True for instances constructed from the same values in
        the same order if `strictfp` is in effect, or if the system architecture guarantees
        `strictfp`-like semantics.)
        """
        ...


    def hashCode(self) -> int:
        """
        
        
        **Note:** This hash code is consistent with exact equality of the calculated statistics,
        including the floating point values. See the note on .equals for details.
        """
        ...


    def toString(self) -> str:
        ...


    @staticmethod
    def meanOf(values: Iterable["Number"]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of the
        values. The count must be non-zero.
        
        The definition of the mean is the same as Stats.mean.

        Arguments
        - values: a series of values, which will be converted to `double` values (this may
            cause loss of precision)

        Raises
        - IllegalArgumentException: if the dataset is empty
        """
        ...


    @staticmethod
    def meanOf(values: Iterator["Number"]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of the
        values. The count must be non-zero.
        
        The definition of the mean is the same as Stats.mean.

        Arguments
        - values: a series of values, which will be converted to `double` values (this may
            cause loss of precision)

        Raises
        - IllegalArgumentException: if the dataset is empty
        """
        ...


    @staticmethod
    def meanOf(*values: Tuple[float, ...]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of the
        values. The count must be non-zero.
        
        The definition of the mean is the same as Stats.mean.

        Arguments
        - values: a series of values

        Raises
        - IllegalArgumentException: if the dataset is empty
        """
        ...


    @staticmethod
    def meanOf(*values: Tuple[int, ...]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of the
        values. The count must be non-zero.
        
        The definition of the mean is the same as Stats.mean.

        Arguments
        - values: a series of values

        Raises
        - IllegalArgumentException: if the dataset is empty
        """
        ...


    @staticmethod
    def meanOf(*values: Tuple[int, ...]) -> float:
        """
        Returns the <a href="http://en.wikipedia.org/wiki/Arithmetic_mean">arithmetic mean</a> of the
        values. The count must be non-zero.
        
        The definition of the mean is the same as Stats.mean.

        Arguments
        - values: a series of values, which will be converted to `double` values (this may
            cause loss of precision for longs of magnitude over 2^53 (slightly over 9e15))

        Raises
        - IllegalArgumentException: if the dataset is empty
        """
        ...


    def toByteArray(self) -> list[int]:
        """
        Gets a byte array representation of this instance.
        
        **Note:** No guarantees are made regarding stability of the representation between
        versions.
        """
        ...


    @staticmethod
    def fromByteArray(byteArray: list[int]) -> "Stats":
        """
        Creates a Stats instance from the given byte representation which was obtained by .toByteArray.
        
        **Note:** No guarantees are made regarding stability of the representation between
        versions.
        """
        ...



"""
Python module generated from Java source file java.util.OptionalDouble

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from java.util.function import DoubleConsumer
from java.util.function import DoubleSupplier
from java.util.function import Supplier
from java.util.stream import DoubleStream
from typing import Any, Callable, Iterable, Tuple


class OptionalDouble:
    """
    A container object which may or may not contain a `double` value.
    If a value is present, `isPresent()` returns `True`. If no
    value is present, the object is considered *empty* and
    `isPresent()` returns `False`.
    
    Additional methods that depend on the presence or absence of a contained
    value are provided, such as .orElse(double) orElse()
    (returns a default value if no value is present) and
    .ifPresent(DoubleConsumer) ifPresent() (performs
    an action if a value is present).
    
    This is a <a href="/java.base/java/lang/doc-files/ValueBased.html">value-based</a>
    class; programmers should treat instances that are
    .equals(Object) equal as interchangeable and should not
    use instances for synchronization, or unpredictable behavior may
    occur. For example, in a future release, synchronization may fail.

    Since
    - 1.8

    Unknown Tags
    - `OptionalDouble` is primarily intended for use as a method return type where
    there is a clear need to represent "no result." A variable whose type is
    `OptionalDouble` should never itself be `null`; it should always point
    to an `OptionalDouble` instance.
    """

    @staticmethod
    def empty() -> "OptionalDouble":
        """
        Returns an empty `OptionalDouble` instance.  No value is present
        for this `OptionalDouble`.

        Returns
        - an empty `OptionalDouble`.

        Unknown Tags
        - Though it may be tempting to do so, avoid testing if an object is empty
        by comparing with `==` or `!=` against instances returned by
        `OptionalDouble.empty()`.  There is no guarantee that it is a singleton.
        Instead, use .isEmpty() or .isPresent().
        """
        ...


    @staticmethod
    def of(value: float) -> "OptionalDouble":
        """
        Returns an `OptionalDouble` describing the given value.

        Arguments
        - value: the value to describe

        Returns
        - an `OptionalDouble` with the value present
        """
        ...


    def getAsDouble(self) -> float:
        """
        If a value is present, returns the value, otherwise throws
        `NoSuchElementException`.

        Returns
        - the value described by this `OptionalDouble`

        Raises
        - NoSuchElementException: if no value is present

        Unknown Tags
        - The preferred alternative to this method is .orElseThrow().
        """
        ...


    def isPresent(self) -> bool:
        """
        If a value is present, returns `True`, otherwise `False`.

        Returns
        - `True` if a value is present, otherwise `False`
        """
        ...


    def isEmpty(self) -> bool:
        """
        If a value is not present, returns `True`, otherwise
        `False`.

        Returns
        - `True` if a value is not present, otherwise `False`

        Since
        - 11
        """
        ...


    def ifPresent(self, action: "DoubleConsumer") -> None:
        """
        If a value is present, performs the given action with the value,
        otherwise does nothing.

        Arguments
        - action: the action to be performed, if a value is present

        Raises
        - NullPointerException: if value is present and the given action is
                `null`
        """
        ...


    def ifPresentOrElse(self, action: "DoubleConsumer", emptyAction: "Runnable") -> None:
        """
        If a value is present, performs the given action with the value,
        otherwise performs the given empty-based action.

        Arguments
        - action: the action to be performed, if a value is present
        - emptyAction: the empty-based action to be performed, if no value is
        present

        Raises
        - NullPointerException: if a value is present and the given action
                is `null`, or no value is present and the given empty-based
                action is `null`.

        Since
        - 9
        """
        ...


    def stream(self) -> "DoubleStream":
        """
        If a value is present, returns a sequential DoubleStream
        containing only that value, otherwise returns an empty
        `DoubleStream`.

        Returns
        - the optional value as a `DoubleStream`

        Since
        - 9

        Unknown Tags
        - This method can be used to transform a `Stream` of optional doubles
        to a `DoubleStream` of present doubles:
        ````Stream<OptionalDouble> os = ..
            DoubleStream s = os.flatMapToDouble(OptionalDouble::stream)````
        """
        ...


    def orElse(self, other: float) -> float:
        """
        If a value is present, returns the value, otherwise returns
        `other`.

        Arguments
        - other: the value to be returned, if no value is present

        Returns
        - the value, if present, otherwise `other`
        """
        ...


    def orElseGet(self, supplier: "DoubleSupplier") -> float:
        """
        If a value is present, returns the value, otherwise returns the result
        produced by the supplying function.

        Arguments
        - supplier: the supplying function that produces a value to be returned

        Returns
        - the value, if present, otherwise the result produced by the
                supplying function

        Raises
        - NullPointerException: if no value is present and the supplying
                function is `null`
        """
        ...


    def orElseThrow(self) -> float:
        """
        If a value is present, returns the value, otherwise throws
        `NoSuchElementException`.

        Returns
        - the value described by this `OptionalDouble`

        Raises
        - NoSuchElementException: if no value is present

        Since
        - 10
        """
        ...


    def orElseThrow(self, exceptionSupplier: "Supplier"["X"]) -> float:
        """
        If a value is present, returns the value, otherwise throws an exception
        produced by the exception supplying function.
        
        Type `<X>`: Type of the exception to be thrown

        Arguments
        - exceptionSupplier: the supplying function that produces an
               exception to be thrown

        Returns
        - the value, if present

        Raises
        - X: if no value is present
        - NullPointerException: if no value is present and the exception
                supplying function is `null`

        Unknown Tags
        - A method reference to the exception constructor with an empty argument
        list can be used as the supplier. For example,
        `IllegalStateException::new`
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Indicates whether some other object is "equal to" this
        `OptionalDouble`. The other object is considered equal if:
        
        - it is also an `OptionalDouble` and;
        - both instances have no value present or;
        - the present values are "equal to" each other via
        `Double.compare() == 0`.

        Arguments
        - obj: an object to be tested for equality

        Returns
        - `True` if the other object is "equal to" this object
                otherwise `False`
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code of the value, if present, otherwise `0`
        (zero) if no value is present.

        Returns
        - hash code value of the present value or `0` if no value is
                present
        """
        ...


    def toString(self) -> str:
        """
        Returns a non-empty string representation of this `OptionalDouble`
        suitable for debugging.  The exact presentation format is unspecified and
        may vary between implementations and versions.

        Returns
        - the string representation of this instance

        Unknown Tags
        - If a value is present the result must include its string representation
        in the result.  Empty and present `OptionalDouble`s must be
        unambiguously differentiable.
        """
        ...



"""
Python module generated from Java source file com.google.common.graph.AbstractValueGraph

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.collect import Maps
from com.google.common.graph import *
from java.util import Optional
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class AbstractValueGraph(AbstractBaseGraph, ValueGraph):
    """
    This class provides a skeletal implementation of ValueGraph. It is recommended to extend
    this class rather than implement ValueGraph directly.
    
    The methods implemented in this class should not be overridden unless the subclass admits a
    more efficient implementation.
    
    Type `<N>`: Node parameter type
    
    Type `<V>`: Value parameter type

    Author(s)
    - James Sexton

    Since
    - 20.0
    """

    def asGraph(self) -> "Graph"["N"]:
        ...


    def edgeValue(self, nodeU: "N", nodeV: "N") -> "Optional"["V"]:
        ...


    def edgeValue(self, endpoints: "EndpointPair"["N"]) -> "Optional"["V"]:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        """
        Returns a string representation of this graph.
        """
        ...



"""
Python module generated from Java source file org.bukkit.generator.BlockPopulator

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Random
from org.bukkit import Chunk
from org.bukkit import World
from org.bukkit.event.world import WorldInitEvent
from org.bukkit.generator import *
from typing import Any, Callable, Iterable, Tuple


class BlockPopulator:
    """
    A block populator is responsible for generating a small area of blocks.
    
    For example, generating glowstone inside the nether or generating dungeons
    full of treasure
    
    A BlockPopulator can be used in combination with a custom ChunkGenerator
    by returning it in the method ChunkGenerator.getDefaultPopulators(World)
    or by adding it manually to the worlds populator list returned by World.getPopulators().
    
    When adding a BlockPopulator manually to a world it is recommended to do so during
    the WorldInitEvent.
    """

    def populate(self, world: "World", random: "Random", source: "Chunk") -> None:
        """
        Populates an area of blocks at or around the given chunk.
        
        The chunks on each side of the specified chunk must already exist; that
        is, there must be one north, east, south and west of the specified
        chunk. The "corner" chunks may not exist, in which scenario the
        populator should record any changes required for those chunks and
        perform the changes when they are ready.

        Arguments
        - world: The world to generate in
        - random: The random generator to use
        - source: The chunk to generate for

        Deprecated
        - Use .populate(WorldInfo, Random, int, int, LimitedRegion)
        """
        ...


    def populate(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int, limitedRegion: "LimitedRegion") -> None:
        """
        Populates an area of blocks at or around the given chunk.
        
        Notes:
        
        This method should **never** attempt to get the Chunk at the passed
        coordinates, as doing so may cause an infinite loop
        
        This method should **never** modify a LimitedRegion at a later
        point of time.
        
        This method **must** be completely thread safe and able to handle
        multiple concurrent callers.
        
        No physics are applied, whether or not it is set to True in
        org.bukkit.block.BlockState.update(boolean, boolean)
        
        **Only** use the org.bukkit.block.BlockState returned by
        LimitedRegion,
        **never** use methods from a World to modify the chunk.

        Arguments
        - worldInfo: The world info of the world to generate in
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk
        - limitedRegion: The chunk region to populate
        """
        ...



"""
Python module generated from Java source file com.google.common.io.CharSource

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Ascii
from com.google.common.base import Optional
from com.google.common.base import Splitter
from com.google.common.collect import AbstractIterator
from com.google.common.collect import ImmutableList
from com.google.common.collect import Lists
from com.google.common.collect import Streams
from com.google.common.io import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import MustBeClosed
from java.io import BufferedReader
from java.io import IOException
from java.io import InputStream
from java.io import Reader
from java.io import StringReader
from java.io import UncheckedIOException
from java.io import Writer
from java.nio.charset import Charset
from java.util import Iterator
from java.util.function import Consumer
from java.util.stream import Stream
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class CharSource:
    """
    A readable source of characters, such as a text file. Unlike a Reader, a `CharSource` is not an open, stateful stream of characters that can be read and closed. Instead,
    it is an immutable *supplier* of `Reader` instances.
    
    `CharSource` provides two kinds of methods:
    
    
      - **Methods that return a reader:** These methods should return a *new*, independent
          instance each time they are called. The caller is responsible for ensuring that the
          returned reader is closed.
      - **Convenience methods:** These are implementations of common operations that are
          typically implemented by opening a reader using one of the methods in the first category,
          doing something and finally closing the reader that was opened.
    
    
    Several methods in this class, such as .readLines(), break the contents of the source
    into lines. Like BufferedReader, these methods break lines on any of `\n`, `\r` or `\r\n`, do not include the line separator in each line and do not consider there to
    be an empty line at the end if the contents are terminated with a line separator.
    
    Any ByteSource containing text encoded with a specific Charset character
    encoding may be viewed as a `CharSource` using ByteSource.asCharSource(Charset).
    
    **Note:** In general, `CharSource` is intended to be used for "file-like" sources
    that provide readers that are:
    
    
      - **Finite:** Many operations, such as .length() and .read(), will either
          block indefinitely or fail if the source creates an infinite reader.
      - **Non-destructive:** A *destructive* reader will consume or otherwise alter the
          source as they are read from it. A source that provides such readers will not be reusable,
          and operations that read from the stream (including .length(), in some
          implementations) will prevent further operations from completing as expected.

    Author(s)
    - Colin Decker

    Since
    - 14.0
    """

    def asByteSource(self, charset: "Charset") -> "ByteSource":
        """
        Returns a ByteSource view of this char source that encodes chars read from this source
        as bytes using the given Charset.
        
        If ByteSource.asCharSource is called on the returned source with the same charset,
        the default implementation of this method will ensure that the original `CharSource` is
        returned, rather than round-trip encoding. Subclasses that override this method should behave
        the same way.

        Since
        - 20.0
        """
        ...


    def openStream(self) -> "Reader":
        """
        Opens a new Reader for reading from this source. This method returns a new, independent
        reader each time it is called.
        
        The caller is responsible for ensuring that the returned reader is closed.

        Raises
        - IOException: if an I/O error occurs while opening the reader
        """
        ...


    def openBufferedStream(self) -> "BufferedReader":
        """
        Opens a new BufferedReader for reading from this source. This method returns a new,
        independent reader each time it is called.
        
        The caller is responsible for ensuring that the returned reader is closed.

        Raises
        - IOException: if an I/O error occurs while of opening the reader
        """
        ...


    def lines(self) -> "Stream"[str]:
        """
        Opens a new Stream for reading text one line at a time from this source. This method
        returns a new, independent stream each time it is called.
        
        The returned stream is lazy and only reads from the source in the terminal operation. If an
        I/O error occurs while the stream is reading from the source or when the stream is closed, an
        UncheckedIOException is thrown.
        
        Like BufferedReader.readLine(), this method considers a line to be a sequence of
        text that is terminated by (but does not include) one of `\r\n`, `\r` or `\n`. If the source's content does not end in a line termination sequence, it is treated as if
        it does.
        
        The caller is responsible for ensuring that the returned stream is closed. For example:
        
        ````try (Stream<String> lines = source.lines()) {
          lines.map(...)
             .filter(...)
             .forEach(...);`
        }```

        Raises
        - IOException: if an I/O error occurs while opening the stream

        Since
        - 22.0
        """
        ...


    def lengthIfKnown(self) -> "Optional"["Long"]:
        """
        Returns the size of this source in chars, if the size can be easily determined without actually
        opening the data stream.
        
        The default implementation returns Optional.absent. Some sources, such as a `CharSequence`, may return a non-absent value. Note that in such cases, it is *possible*
        that this method will return a different number of chars than would be returned by reading all
        of the chars.
        
        Additionally, for mutable sources such as `StringBuilder`s, a subsequent read may
        return a different number of chars if the contents are changed.

        Since
        - 19.0
        """
        ...


    def length(self) -> int:
        """
        Returns the length of this source in chars, even if doing so requires opening and traversing an
        entire stream. To avoid a potentially expensive operation, see .lengthIfKnown.
        
        The default implementation calls .lengthIfKnown and returns the value if present. If
        absent, it will fall back to a heavyweight operation that will open a stream, Reader.skip(long) skip to the end of the stream, and return the total number of chars that
        were skipped.
        
        Note that for sources that implement .lengthIfKnown to provide a more efficient
        implementation, it is *possible* that this method will return a different number of chars
        than would be returned by reading all of the chars.
        
        In either case, for mutable sources such as files, a subsequent read may return a different
        number of chars if the contents are changed.

        Raises
        - IOException: if an I/O error occurs while reading the length of this source

        Since
        - 19.0
        """
        ...


    def copyTo(self, appendable: "Appendable") -> int:
        """
        Appends the contents of this source to the given Appendable (such as a Writer).
        Does not close `appendable` if it is `Closeable`.

        Returns
        - the number of characters copied

        Raises
        - IOException: if an I/O error occurs while reading from this source or writing to `appendable`
        """
        ...


    def copyTo(self, sink: "CharSink") -> int:
        """
        Copies the contents of this source to the given sink.

        Returns
        - the number of characters copied

        Raises
        - IOException: if an I/O error occurs while reading from this source or writing to `sink`
        """
        ...


    def read(self) -> str:
        """
        Reads the contents of this source as a string.

        Raises
        - IOException: if an I/O error occurs while reading from this source
        """
        ...


    def readFirstLine(self) -> str:
        """
        Reads the first line of this source as a string. Returns `null` if this source is empty.
        
        Like BufferedReader.readLine(), this method considers a line to be a sequence of
        text that is terminated by (but does not include) one of `\r\n`, `\r` or `\n`. If the source's content does not end in a line termination sequence, it is treated as if
        it does.

        Raises
        - IOException: if an I/O error occurs while reading from this source
        """
        ...


    def readLines(self) -> "ImmutableList"[str]:
        """
        Reads all the lines of this source as a list of strings. The returned list will be empty if
        this source is empty.
        
        Like BufferedReader.readLine(), this method considers a line to be a sequence of
        text that is terminated by (but does not include) one of `\r\n`, `\r` or `\n`. If the source's content does not end in a line termination sequence, it is treated as if
        it does.

        Raises
        - IOException: if an I/O error occurs while reading from this source
        """
        ...


    def readLines(self, processor: "LineProcessor"["T"]) -> "T":
        """
        Reads lines of text from this source, processing each line as it is read using the given LineProcessor processor. Stops when all lines have been processed or the processor returns
        `False` and returns the result produced by the processor.
        
        Like BufferedReader.readLine(), this method considers a line to be a sequence of
        text that is terminated by (but does not include) one of `\r\n`, `\r` or `\n`. If the source's content does not end in a line termination sequence, it is treated as if
        it does.

        Raises
        - IOException: if an I/O error occurs while reading from this source or if `processor` throws an `IOException`

        Since
        - 16.0
        """
        ...


    def forEachLine(self, action: "Consumer"[str]) -> None:
        """
        Reads all lines of text from this source, running the given `action` for each line as it
        is read.
        
        Like BufferedReader.readLine(), this method considers a line to be a sequence of
        text that is terminated by (but does not include) one of `\r\n`, `\r` or `\n`. If the source's content does not end in a line termination sequence, it is treated as if
        it does.

        Raises
        - IOException: if an I/O error occurs while reading from this source or if `action`
            throws an `UncheckedIOException`

        Since
        - 22.0
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns whether the source has zero chars. The default implementation first checks .lengthIfKnown, returning True if it's known to be zero and False if it's known to be
        non-zero. If the length is not known, it falls back to opening a stream and checking for EOF.
        
        Note that, in cases where `lengthIfKnown` returns zero, it is *possible* that
        chars are actually available for reading. This means that a source may return `True` from
        `isEmpty()` despite having readable content.

        Raises
        - IOException: if an I/O error occurs

        Since
        - 15.0
        """
        ...


    @staticmethod
    def concat(sources: Iterable["CharSource"]) -> "CharSource":
        """
        Concatenates multiple CharSource instances into a single source. Streams returned from
        the source will contain the concatenated data from the streams of the underlying sources.
        
        Only one underlying stream will be open at a time. Closing the concatenated stream will
        close the open underlying stream.

        Arguments
        - sources: the sources to concatenate

        Returns
        - a `CharSource` containing the concatenated data

        Since
        - 15.0
        """
        ...


    @staticmethod
    def concat(sources: Iterator["CharSource"]) -> "CharSource":
        """
        Concatenates multiple CharSource instances into a single source. Streams returned from
        the source will contain the concatenated data from the streams of the underlying sources.
        
        Only one underlying stream will be open at a time. Closing the concatenated stream will
        close the open underlying stream.
        
        Note: The input `Iterator` will be copied to an `ImmutableList` when this method
        is called. This will fail if the iterator is infinite and may cause problems if the iterator
        eagerly fetches data for each source when iterated (rather than producing sources that only
        load data through their streams). Prefer using the .concat(Iterable) overload if
        possible.

        Arguments
        - sources: the sources to concatenate

        Returns
        - a `CharSource` containing the concatenated data

        Raises
        - NullPointerException: if any of `sources` is `null`

        Since
        - 15.0
        """
        ...


    @staticmethod
    def concat(*sources: Tuple["CharSource", ...]) -> "CharSource":
        """
        Concatenates multiple CharSource instances into a single source. Streams returned from
        the source will contain the concatenated data from the streams of the underlying sources.
        
        Only one underlying stream will be open at a time. Closing the concatenated stream will
        close the open underlying stream.

        Arguments
        - sources: the sources to concatenate

        Returns
        - a `CharSource` containing the concatenated data

        Raises
        - NullPointerException: if any of `sources` is `null`

        Since
        - 15.0
        """
        ...


    @staticmethod
    def wrap(charSequence: "CharSequence") -> "CharSource":
        """
        Returns a view of the given character sequence as a CharSource. The behavior of the
        returned `CharSource` and any `Reader` instances created by it is unspecified if
        the `charSequence` is mutated while it is being read, so don't do that.

        Since
        - 15.0 (since 14.0 as `CharStreams.asCharSource(String)`)
        """
        ...


    @staticmethod
    def empty() -> "CharSource":
        """
        Returns an immutable CharSource that contains no characters.

        Since
        - 15.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.command.defaults.VersionCommand

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Charsets
from com.google.common.base import Preconditions
from com.google.common.collect import ImmutableList
from com.google.common.io import Resources
from com.google.gson import Gson
from com.google.gson import JsonObject
from com.google.gson import JsonSyntaxException
from java.io import BufferedReader
from java.io import IOException
from java.net import URL
from java.net import URLEncoder
from java.util import Arrays
from java.util import Locale
from java.util.concurrent.locks import ReentrantLock
from org.bukkit import Bukkit
from org.bukkit import ChatColor
from org.bukkit.command import CommandSender
from org.bukkit.command.defaults import *
from org.bukkit.plugin import Plugin
from org.bukkit.plugin import PluginDescriptionFile
from org.bukkit.util import StringUtil
from typing import Any, Callable, Iterable, Tuple


class VersionCommand(BukkitCommand):

    def __init__(self, name: str):
        ...


    def execute(self, sender: "CommandSender", currentAlias: str, args: list[str]) -> bool:
        ...


    def tabComplete(self, sender: "CommandSender", alias: str, args: list[str]) -> list[str]:
        ...



"""
Python module generated from Java source file com.google.common.io.Closer

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.io import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import Closeable
from java.io import IOException
from java.util import ArrayDeque
from java.util import Deque
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Closer(Closeable):

    @staticmethod
    def create() -> "Closer":
        """
        Creates a new Closer.
        """
        ...


    def register(self, closeable: "C") -> "C":
        ...


    def rethrow(self, e: "Throwable") -> "RuntimeException":
        """
        Stores the given throwable and rethrows it. It will be rethrown as is if it is an `IOException`, `RuntimeException` or `Error`. Otherwise, it will be rethrown wrapped
        in a `RuntimeException`. **Note:** Be sure to declare all of the checked exception
        types your try block can throw when calling an overload of this method so as to avoid losing
        the original exception type.
        
        This method always throws, and as such should be called as `throw closer.rethrow(e);`
        to ensure the compiler knows that it will throw.

        Returns
        - this method does not return; it always throws

        Raises
        - IOException: when the given throwable is an IOException
        """
        ...


    def rethrow(self, e: "Throwable", declaredType: type["X"]) -> "RuntimeException":
        """
        Stores the given throwable and rethrows it. It will be rethrown as is if it is an `IOException`, `RuntimeException`, `Error` or a checked exception of the given type.
        Otherwise, it will be rethrown wrapped in a `RuntimeException`. **Note:** Be sure to
        declare all of the checked exception types your try block can throw when calling an overload of
        this method so as to avoid losing the original exception type.
        
        This method always throws, and as such should be called as `throw closer.rethrow(e,
        ...);` to ensure the compiler knows that it will throw.

        Returns
        - this method does not return; it always throws

        Raises
        - IOException: when the given throwable is an IOException
        - X: when the given throwable is of the declared type X
        """
        ...


    def rethrow(self, e: "Throwable", declaredType1: type["X1"], declaredType2: type["X2"]) -> "RuntimeException":
        """
        Stores the given throwable and rethrows it. It will be rethrown as is if it is an `IOException`, `RuntimeException`, `Error` or a checked exception of either of the
        given types. Otherwise, it will be rethrown wrapped in a `RuntimeException`. **Note:**
        Be sure to declare all of the checked exception types your try block can throw when calling an
        overload of this method so as to avoid losing the original exception type.
        
        This method always throws, and as such should be called as `throw closer.rethrow(e,
        ...);` to ensure the compiler knows that it will throw.

        Returns
        - this method does not return; it always throws

        Raises
        - IOException: when the given throwable is an IOException
        - X1: when the given throwable is of the declared type X1
        - X2: when the given throwable is of the declared type X2
        """
        ...


    def close(self) -> None:
        """
        Closes all `Closeable` instances that have been added to this `Closer`. If an
        exception was thrown in the try block and passed to one of the `exceptionThrown` methods,
        any exceptions thrown when attempting to close a closeable will be suppressed. Otherwise, the
        *first* exception to be thrown from an attempt to close a closeable will be thrown and any
        additional exceptions that are thrown after that will be suppressed.
        """
        ...



"""
Python module generated from Java source file org.bukkit.profile.PlayerTextures

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.net import URL
from org.bukkit.profile import *
from typing import Any, Callable, Iterable, Tuple


class PlayerTextures:
    """
    Provides access to the textures stored inside a PlayerProfile.
    
    Modifying the textures immediately invalidates and clears any previously
    present attributes that are specific to official player profiles, such as the
    .getTimestamp() timestamp and .isSigned() signature.
    """

    def isEmpty(self) -> bool:
        """
        Checks if the profile stores no textures.

        Returns
        - `True` if the profile stores no textures
        """
        ...


    def clear(self) -> None:
        """
        Clears the textures.
        """
        ...


    def getSkin(self) -> "URL":
        """
        Gets the URL that points to the player's skin.

        Returns
        - the URL of the player's skin, or `null` if not set
        """
        ...


    def setSkin(self, skinUrl: "URL") -> None:
        """
        Sets the player's skin to the specified URL, and the skin model to
        SkinModel.CLASSIC.
        
        The URL **must** point to the Minecraft texture server. Example URL:
        ```
        http://textures.minecraft.net/texture/b3fbd454b599df593f57101bfca34e67d292a8861213d2202bb575da7fd091ac
        ```

        Arguments
        - skinUrl: the URL of the player's skin, or `null` to
        unset it
        """
        ...


    def setSkin(self, skinUrl: "URL", skinModel: "SkinModel") -> None:
        """
        Sets the player's skin and SkinModel.
        
        The URL **must** point to the Minecraft texture server. Example URL:
        ```
        http://textures.minecraft.net/texture/b3fbd454b599df593f57101bfca34e67d292a8861213d2202bb575da7fd091ac
        ```
        
        A skin model of `null` results in SkinModel.CLASSIC to
        be used.

        Arguments
        - skinUrl: the URL of the player's skin, or `null` to
        unset it
        - skinModel: the skin model, ignored if the skin URL is
        `null`
        """
        ...


    def getSkinModel(self) -> "SkinModel":
        """
        Gets the model of the player's skin.
        
        This returns SkinModel.CLASSIC if no skin is set.

        Returns
        - the model of the player's skin
        """
        ...


    def getCape(self) -> "URL":
        """
        Gets the URL that points to the player's cape.

        Returns
        - the URL of the player's cape, or `null` if not set
        """
        ...


    def setCape(self, capeUrl: "URL") -> None:
        """
        Sets the URL that points to the player's cape.
        
        The URL **must** point to the Minecraft texture server. Example URL:
        ```
        http://textures.minecraft.net/texture/2340c0e03dd24a11b15a8b33c2a7e9e32abb2051b2481d0ba7defd635ca7a933
        ```

        Arguments
        - capeUrl: the URL of the player's cape, or `null` to
        unset it
        """
        ...


    def getTimestamp(self) -> int:
        """
        Gets the timestamp at which the profile was last updated.

        Returns
        - the timestamp, or `0` if unknown
        """
        ...


    def isSigned(self) -> bool:
        """
        Checks if the textures are signed and the signature is valid.

        Returns
        - `True` if the textures are signed and the signature is
        valid
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.constructor.CustomClassLoaderConstructor

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml import LoaderOptions
from org.yaml.snakeyaml.constructor import *
from typing import Any, Callable, Iterable, Tuple


class CustomClassLoaderConstructor(Constructor):
    """
    Construct instances with a custom Class Loader.
    """

    def __init__(self, loader: "ClassLoader", loadingConfig: "LoaderOptions"):
        """
        Create

        Arguments
        - loader: - the class loader to find the class definition
        - loadingConfig: - options
        """
        ...


    def __init__(self, theRoot: type["Object"], theLoader: "ClassLoader", loadingConfig: "LoaderOptions"):
        """
        Create

        Arguments
        - loadingConfig: - options
        - theRoot: - the class to instantiate
        - theLoader: - the class loader to find the class definition
        """
        ...



from .InetAddress import InetAddress
from .URI import URI
from .URISyntaxException import URISyntaxException
from .URL import URL
from .MalformedURLException import MalformedURLException
from .URLClassLoader import URLClassLoader
from .Inet4Address import Inet4Address
from .Inet6Address import Inet6Address
from .NetworkInterface import NetworkInterface
from .SocketException import SocketException
from .UnknownHostException import UnknownHostException
from .URLDecoder import URLDecoder
from .URLConnection import URLConnection
from .InetSocketAddress import InetSocketAddress
from .SocketAddress import SocketAddress
from .URLEncoder import URLEncoder
from .HttpURLConnection import HttpURLConnection


"""
Python module generated from Java source file com.google.gson.GsonBuilder

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import InlineMe
from com.google.gson import *
from com.google.gson.annotations import Since
from com.google.gson.annotations import Until
from com.google.gson.internal import $Gson$Preconditions
from com.google.gson.internal import Excluder
from com.google.gson.internal.bind import DefaultDateTypeAdapter
from com.google.gson.internal.bind import TreeTypeAdapter
from com.google.gson.internal.bind import TypeAdapters
from com.google.gson.internal.sql import SqlTypesSupport
from com.google.gson.reflect import TypeToken
from com.google.gson.stream import JsonReader
from com.google.gson.stream import JsonWriter
from java.lang.reflect import Type
from java.text import DateFormat
from java.text import SimpleDateFormat
from java.util import ArrayDeque
from java.util import Collections
from java.util import Date
from java.util import Objects
from typing import Any, Callable, Iterable, Tuple


class GsonBuilder:
    """
    Use this builder to construct a Gson instance when you need to set configuration options
    other than the default. For Gson with default configuration, it is simpler to use `new Gson()`. `GsonBuilder` is best used by creating it, and then invoking its various
    configuration methods, and finally calling create.
    
    The following example shows how to use the `GsonBuilder` to construct a Gson instance:
    
    ```
    Gson gson = new GsonBuilder()
        .registerTypeAdapter(Id.class, new IdTypeAdapter())
        .enableComplexMapKeySerialization()
        .serializeNulls()
        .setDateFormat(DateFormat.LONG, DateFormat.LONG)
        .setFieldNamingPolicy(FieldNamingPolicy.UPPER_CAMEL_CASE)
        .setPrettyPrinting()
        .setVersion(1.0)
        .create();
    ```
    
    Notes:
    
    
      - The order of invocation of configuration methods does not matter.
      - The default serialization of Date and its subclasses in Gson does not contain
          time-zone information. So, if you are using date/time instances, use `GsonBuilder`
          and its `setDateFormat` methods.
      - By default no explicit Strictness is set; some of the Gson methods behave
          as if Strictness.LEGACY_STRICT was used whereas others behave as if Strictness.LENIENT was used. Prefer explicitly setting a strictness with .setStrictness(Strictness) to avoid this legacy behavior.

    Author(s)
    - Jesse Wilson
    """

    def __init__(self):
        """
        Creates a GsonBuilder instance that can be used to build Gson with various configuration
        settings. GsonBuilder follows the builder pattern, and it is typically used by first invoking
        various configuration methods to set desired options, and finally calling .create().
        """
        ...


    def setVersion(self, version: float) -> "GsonBuilder":
        """
        Configures Gson to enable versioning support. Versioning support works based on the annotation
        types Since and Until. It allows including or excluding fields and classes
        based on the specified version. See the documentation of these annotation types for more
        information.
        
        By default versioning support is disabled and usage of `@Since` and `@Until` has
        no effect.

        Arguments
        - version: the version number to use.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Raises
        - IllegalArgumentException: if the version number is NaN or negative

        See
        - Until
        """
        ...


    def excludeFieldsWithModifiers(self, *modifiers: Tuple[int, ...]) -> "GsonBuilder":
        """
        Configures Gson to excludes all class fields that have the specified modifiers. By default,
        Gson will exclude all fields marked `transient` or `static`. This method will
        override that behavior.
        
        This is a convenience method which behaves as if an ExclusionStrategy which excludes
        these fields was .setExclusionStrategies(ExclusionStrategy...) registered with this
        builder.

        Arguments
        - modifiers: the field modifiers. You must use the modifiers specified in the java.lang.reflect.Modifier class. For example, java.lang.reflect.Modifier.TRANSIENT, java.lang.reflect.Modifier.STATIC.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern
        """
        ...


    def generateNonExecutableJson(self) -> "GsonBuilder":
        """
        Makes the output JSON non-executable in Javascript by prefixing the generated JSON with some
        special text. This prevents attacks from third-party sites through script sourcing. See <a
        href="http://code.google.com/p/google-gson/issues/detail?id=42">Gson Issue 42</a> for details.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.3
        """
        ...


    def excludeFieldsWithoutExposeAnnotation(self) -> "GsonBuilder":
        """
        Configures Gson to exclude all fields from consideration for serialization and deserialization
        that do not have the com.google.gson.annotations.Expose annotation.
        
        This is a convenience method which behaves as if an ExclusionStrategy which excludes
        these fields was .setExclusionStrategies(ExclusionStrategy...) registered with this
        builder.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern
        """
        ...


    def serializeNulls(self) -> "GsonBuilder":
        """
        Configures Gson to serialize null fields. By default, Gson omits all fields that are null
        during serialization.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.2
        """
        ...


    def enableComplexMapKeySerialization(self) -> "GsonBuilder":
        """
        Configures Gson to serialize `Map` objects with complex keys as JSON arrays. Enabling
        this feature will only change the serialized form if the map key is a complex type (i.e.
        non-primitive) in its <strong>serialized</strong> JSON form. The default implementation of map
        serialization uses `toString()` on the key; however, when this is called then one of the
        following cases apply:
        
        **Maps as JSON objects**
        
        For this case, assume that a type adapter is registered to serialize and deserialize some
        `Point` class, which contains an x and y coordinate, to/from the JSON Primitive string
        value `"(x,y)"`. The Java map would then be serialized as a JsonObject.
        
        Below is an example:
        
        ````Gson gson = new GsonBuilder()
            .register(Point.class, new MyPointTypeAdapter())
            .enableComplexMapKeySerialization()
            .create();
        
        Map<Point, String> original = new LinkedHashMap<>();
        original.put(new Point(5, 6), "a");
        original.put(new Point(8, 8), "b");
        System.out.println(gson.toJson(original, type));````
        
        The above code prints this JSON object:
        
        ````{
          "(5,6)": "a",
          "(8,8)": "b"`
        }```
        
        **Maps as JSON arrays**
        
        For this case, assume that a type adapter was NOT registered for some `Point` class,
        but rather the default Gson serialization is applied. In this case, some `new Point(2,3)`
        would serialize as `{"x":2,"y":3`}.
        
        Given the assumption above, a `Map<Point, String>` will be serialized as an array of
        arrays (can be viewed as an entry set of pairs).
        
        Below is an example of serializing complex types as JSON arrays:
        
        ````Gson gson = new GsonBuilder()
            .enableComplexMapKeySerialization()
            .create();
        
        Map<Point, String> original = new LinkedHashMap<>();
        original.put(new Point(5, 6), "a");
        original.put(new Point(8, 8), "b");
        System.out.println(gson.toJson(original, type));````
        
        The JSON output would look as follows:
        
        ````[
          [
            {
              "x": 5,
              "y": 6`,
            "a"
          ],
          [
            {
              "x": 8,
              "y": 8
            },
            "b"
          ]
        ]
        }```

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.7
        """
        ...


    def disableInnerClassSerialization(self) -> "GsonBuilder":
        """
        Configures Gson to exclude inner classes (= non-`static` nested classes) during
        serialization and deserialization. This is a convenience method which behaves as if an ExclusionStrategy which excludes inner classes was .setExclusionStrategies(ExclusionStrategy...) registered with this builder. This means inner
        classes will be serialized as JSON `null`, and will be deserialized as Java `null`
        with their JSON data being ignored. And fields with an inner class as type will be ignored
        during serialization and deserialization.
        
        By default Gson serializes and deserializes inner classes, but ignores references to the
        enclosing instance. Deserialization might not be possible at all when .disableJdkUnsafe() is used (and no custom InstanceCreator is registered), or it can
        lead to unexpected `NullPointerException`s when the deserialized instance is used
        afterwards.
        
        In general using inner classes with Gson should be avoided; they should be converted to
        `static` nested classes if possible.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.3
        """
        ...


    def setLongSerializationPolicy(self, serializationPolicy: "LongSerializationPolicy") -> "GsonBuilder":
        """
        Configures Gson to apply a specific serialization policy for `Long` and `long`
        objects.

        Arguments
        - serializationPolicy: the particular policy to use for serializing longs.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.3
        """
        ...


    def setFieldNamingPolicy(self, namingConvention: "FieldNamingPolicy") -> "GsonBuilder":
        """
        Configures Gson to apply a specific naming policy to an object's fields during serialization
        and deserialization.
        
        This method just delegates to .setFieldNamingStrategy(FieldNamingStrategy).
        """
        ...


    def setFieldNamingStrategy(self, fieldNamingStrategy: "FieldNamingStrategy") -> "GsonBuilder":
        """
        Configures Gson to apply a specific naming strategy to an object's fields during serialization
        and deserialization.
        
        The created Gson instance might only use the field naming strategy once for a field and
        cache the result. It is not guaranteed that the strategy will be used again every time the
        value of a field is serialized or deserialized.

        Arguments
        - fieldNamingStrategy: the naming strategy to apply to the fields

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.3
        """
        ...


    def setObjectToNumberStrategy(self, objectToNumberStrategy: "ToNumberStrategy") -> "GsonBuilder":
        """
        Configures Gson to apply a specific number strategy during deserialization of Object.

        Arguments
        - objectToNumberStrategy: the actual object-to-number strategy

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        See
        - ToNumberPolicy.DOUBLE The default object-to-number strategy

        Since
        - 2.8.9
        """
        ...


    def setNumberToNumberStrategy(self, numberToNumberStrategy: "ToNumberStrategy") -> "GsonBuilder":
        """
        Configures Gson to apply a specific number strategy during deserialization of Number.

        Arguments
        - numberToNumberStrategy: the actual number-to-number strategy

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        See
        - ToNumberPolicy.LAZILY_PARSED_NUMBER The default number-to-number strategy

        Since
        - 2.8.9
        """
        ...


    def setExclusionStrategies(self, *strategies: Tuple["ExclusionStrategy", ...]) -> "GsonBuilder":
        """
        Configures Gson to apply a set of exclusion strategies during both serialization and
        deserialization. Each of the `strategies` will be applied as a disjunction rule. This
        means that if one of the `strategies` suggests that a field (or class) should be skipped
        then that field (or object) is skipped during serialization/deserialization. The strategies are
        added to the existing strategies (if any); the existing strategies are not replaced.
        
        Fields are excluded for serialization and deserialization when ExclusionStrategy.shouldSkipField(FieldAttributes) shouldSkipField returns `True`, or
        when ExclusionStrategy.shouldSkipClass(Class) shouldSkipClass returns `True` for
        the field type. Gson behaves as if the field did not exist; its value is not serialized and on
        deserialization if a JSON member with this name exists it is skipped by default.
        When objects of an excluded type (as determined by ExclusionStrategy.shouldSkipClass(Class) shouldSkipClass) are serialized a JSON null is
        written to output, and when deserialized the JSON value is skipped and `null` is
        returned.
        
        The created Gson instance might only use an exclusion strategy once for a field or class and
        cache the result. It is not guaranteed that the strategy will be used again every time the
        value of a field or a class is serialized or deserialized.

        Arguments
        - strategies: the set of strategy object to apply during object (de)serialization.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.4
        """
        ...


    def addSerializationExclusionStrategy(self, strategy: "ExclusionStrategy") -> "GsonBuilder":
        """
        Configures Gson to apply the passed in exclusion strategy during serialization. If this method
        is invoked numerous times with different exclusion strategy objects then the exclusion
        strategies that were added will be applied as a disjunction rule. This means that if one of the
        added exclusion strategies suggests that a field (or class) should be skipped then that field
        (or object) is skipped during its serialization.
        
        See the documentation of .setExclusionStrategies(ExclusionStrategy...) for a
        detailed description of the effect of exclusion strategies.

        Arguments
        - strategy: an exclusion strategy to apply during serialization.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.7
        """
        ...


    def addDeserializationExclusionStrategy(self, strategy: "ExclusionStrategy") -> "GsonBuilder":
        """
        Configures Gson to apply the passed in exclusion strategy during deserialization. If this
        method is invoked numerous times with different exclusion strategy objects then the exclusion
        strategies that were added will be applied as a disjunction rule. This means that if one of the
        added exclusion strategies suggests that a field (or class) should be skipped then that field
        (or object) is skipped during its deserialization.
        
        See the documentation of .setExclusionStrategies(ExclusionStrategy...) for a
        detailed description of the effect of exclusion strategies.

        Arguments
        - strategy: an exclusion strategy to apply during deserialization.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.7
        """
        ...


    def setPrettyPrinting(self) -> "GsonBuilder":
        """
        Configures Gson to output JSON that fits in a page for pretty printing. This option only
        affects JSON serialization.
        
        This is a convenience method which simply calls .setFormattingStyle(FormattingStyle)
        with FormattingStyle.PRETTY.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern
        """
        ...


    def setFormattingStyle(self, formattingStyle: "FormattingStyle") -> "GsonBuilder":
        """
        Configures Gson to output JSON that uses a certain kind of formatting style (for example
        newline and indent). This option only affects JSON serialization. By default Gson produces
        compact JSON output without any formatting.

        Arguments
        - formattingStyle: the formatting style to use.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 2.11.0
        """
        ...


    def setLenient(self) -> "GsonBuilder":
        """
        Sets the strictness of this builder to Strictness.LENIENT.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern.

        See
        - .setStrictness(Strictness)

        Deprecated
        - This method is equivalent to calling .setStrictness(Strictness) with Strictness.LENIENT: `setStrictness(Strictness.LENIENT)`
        """
        ...


    def setStrictness(self, strictness: "Strictness") -> "GsonBuilder":
        """
        Sets the strictness of this builder to the provided parameter.
        
        This changes how strict the <a href="https://www.ietf.org/rfc/rfc8259.txt">RFC 8259 JSON
        specification</a> is enforced when parsing or writing JSON. For details on this, refer to
        JsonReader.setStrictness(Strictness) and JsonWriter.setStrictness(Strictness).

        Arguments
        - strictness: the new strictness mode. May not be `null`.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern.

        See
        - JsonWriter.setStrictness(Strictness)

        Since
        - 2.11.0
        """
        ...


    def disableHtmlEscaping(self) -> "GsonBuilder":
        """
        By default, Gson escapes HTML characters such as &lt; &gt; etc. Use this option to configure
        Gson to pass-through HTML characters as is.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.3
        """
        ...


    def setDateFormat(self, pattern: str) -> "GsonBuilder":
        """
        Configures Gson to serialize `Date` objects according to the pattern provided. You can
        call this method or .setDateFormat(int, int) multiple times, but only the last
        invocation will be used to decide the serialization format.
        
        The date format will be used to serialize and deserialize java.util.Date and in case
        the `java.sql` module is present, also java.sql.Timestamp and java.sql.Date.
        
        Note that this pattern must abide by the convention provided by `SimpleDateFormat`
        class. See the documentation in SimpleDateFormat for more information on valid date and
        time patterns.

        Arguments
        - pattern: the pattern that dates will be serialized/deserialized to/from; can be `null` to reset the pattern

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Raises
        - IllegalArgumentException: if the pattern is invalid

        Since
        - 1.2
        """
        ...


    def setDateFormat(self, dateStyle: int) -> "GsonBuilder":
        """
        Configures Gson to serialize `Date` objects according to the date style value provided.
        You can call this method or .setDateFormat(String) multiple times, but only the last
        invocation will be used to decide the serialization format. This methods leaves the current
        'time style' unchanged.
        
        Note that this style value should be one of the predefined constants in the DateFormat class, such as DateFormat.MEDIUM. See the documentation of the DateFormat class for more information on the valid style constants.

        Arguments
        - dateStyle: the predefined date style that date objects will be serialized/deserialized
            to/from

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Raises
        - IllegalArgumentException: if the style is invalid

        Since
        - 1.2

        Deprecated
        - Counterintuitively, despite this method taking only a 'date style' Gson will use a
            format which includes both date and time, with the 'time style' being the last value set by
            .setDateFormat(int, int). Therefore prefer using .setDateFormat(int, int)
            and explicitly provide the desired 'time style'.
        """
        ...


    def setDateFormat(self, dateStyle: int, timeStyle: int) -> "GsonBuilder":
        """
        Configures Gson to serialize `Date` objects according to the style value provided. You
        can call this method or .setDateFormat(String) multiple times, but only the last
        invocation will be used to decide the serialization format.
        
        Note that this style value should be one of the predefined constants in the DateFormat class, such as DateFormat.MEDIUM. See the documentation of the DateFormat class for more information on the valid style constants.

        Arguments
        - dateStyle: the predefined date style that date objects will be serialized/deserialized
            to/from
        - timeStyle: the predefined style for the time portion of the date objects

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Raises
        - IllegalArgumentException: if the style values are invalid

        Since
        - 1.2
        """
        ...


    def registerTypeAdapter(self, type: "Type", typeAdapter: "Object") -> "GsonBuilder":
        """
        Configures Gson for custom serialization or deserialization. This method combines the
        registration of an TypeAdapter, InstanceCreator, JsonSerializer, and a
        JsonDeserializer. It is best used when a single object `typeAdapter` implements
        all the required interfaces for custom serialization with Gson. If a type adapter was
        previously registered for the specified `type`, it is overwritten.
        
        This registers the type specified and no other types: you must manually register related
        types! For example, applications registering `boolean.class` should also register `Boolean.class`. And when registering an adapter for a class which has subclasses, you might
        also want to register the adapter for subclasses, or use .registerTypeHierarchyAdapter(Class, Object) instead.
        
        JsonSerializer and JsonDeserializer are made "`null`-safe". This means
        when trying to serialize `null`, Gson will write a JSON `null` and the serializer
        is not called. Similarly when deserializing a JSON `null`, Gson will emit `null`
        without calling the deserializer. If it is desired to handle `null` values, a TypeAdapter should be used instead.

        Arguments
        - type: the type definition for the type adapter being registered
        - typeAdapter: This object must implement at least one of the TypeAdapter, InstanceCreator, JsonSerializer, and a JsonDeserializer interfaces.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Raises
        - IllegalArgumentException: if the type adapter being registered is for `Object`
            class or JsonElement or any of its subclasses

        See
        - .registerTypeHierarchyAdapter(Class, Object)
        """
        ...


    def registerTypeAdapterFactory(self, factory: "TypeAdapterFactory") -> "GsonBuilder":
        """
        Registers a factory for type adapters. Registering a factory is useful when the type adapter
        needs to be configured based on the type of the field being processed. Gson is designed to
        handle a large number of factories, so you should consider registering them to be at par with
        registering an individual type adapter.
        
        The created Gson instance might only use the factory once to create an adapter for a
        specific type and cache the result. It is not guaranteed that the factory will be used again
        every time the type is serialized or deserialized.

        Since
        - 2.1
        """
        ...


    def registerTypeHierarchyAdapter(self, baseType: type[Any], typeAdapter: "Object") -> "GsonBuilder":
        """
        Configures Gson for custom serialization or deserialization for an inheritance type hierarchy.
        This method combines the registration of a TypeAdapter, JsonSerializer and a
        JsonDeserializer. If a type adapter was previously registered for the specified type
        hierarchy, it is overridden. If a type adapter is registered for a specific type in the type
        hierarchy, it will be invoked instead of the one registered for the type hierarchy.

        Arguments
        - baseType: the class definition for the type adapter being registered for the base class
            or interface
        - typeAdapter: This object must implement at least one of TypeAdapter, JsonSerializer or JsonDeserializer interfaces.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Raises
        - IllegalArgumentException: if the type adapter being registered is for JsonElement or any of its subclasses

        Since
        - 1.7
        """
        ...


    def serializeSpecialFloatingPointValues(self) -> "GsonBuilder":
        """
        Section 6 of <a href="https://www.ietf.org/rfc/rfc8259.txt">JSON specification</a> disallows
        special double values (NaN, Infinity, -Infinity). However, <a
        href="http://www.ecma-international.org/publications/files/ECMA-ST/Ecma-262.pdf">Javascript
        specification</a> (see section 4.3.20, 4.3.22, 4.3.23) allows these values as valid Javascript
        values. Moreover, most JavaScript engines will accept these special values in JSON without
        problem. So, at a practical level, it makes sense to accept these values as valid JSON even
        though JSON specification disallows them.
        
        Gson always accepts these special values during deserialization. However, it outputs
        strictly compliant JSON. Hence, if it encounters a float value Float.NaN, Float.POSITIVE_INFINITY, Float.NEGATIVE_INFINITY, or a double value Double.NaN, Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, it will throw
        an IllegalArgumentException. This method provides a way to override the default
        behavior when you know that the JSON receiver will be able to handle these special values.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 1.3
        """
        ...


    def disableJdkUnsafe(self) -> "GsonBuilder":
        """
        Disables usage of JDK's `sun.misc.Unsafe`.
        
        By default Gson uses `Unsafe` to create instances of classes which don't have a
        no-args constructor. However, `Unsafe` might not be available for all Java runtimes. For
        example Android does not provide `Unsafe`, or only with limited functionality.
        Additionally `Unsafe` creates instances without executing any constructor or initializer
        block, or performing initialization of field values. This can lead to surprising and difficult
        to debug errors. Therefore, to get reliable behavior regardless of which runtime is used, and
        to detect classes which cannot be deserialized in an early stage of development, this method
        allows disabling usage of `Unsafe`.

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 2.9.0
        """
        ...


    def addReflectionAccessFilter(self, filter: "ReflectionAccessFilter") -> "GsonBuilder":
        """
        Adds a reflection access filter. A reflection access filter prevents Gson from using reflection
        for the serialization and deserialization of certain classes. The logic in the filter specifies
        which classes those are.
        
        Filters will be invoked in reverse registration order, that is, the most recently added
        filter will be invoked first.
        
        By default Gson has no filters configured and will try to use reflection for all classes for
        which no TypeAdapter has been registered, and for which no built-in Gson `TypeAdapter` exists.
        
        The created Gson instance might only use an access filter once for a class or its members
        and cache the result. It is not guaranteed that the filter will be used again every time a
        class or its members are accessed during serialization or deserialization.

        Arguments
        - filter: filter to add

        Returns
        - a reference to this `GsonBuilder` object to fulfill the "Builder" pattern

        Since
        - 2.9.1
        """
        ...


    def create(self) -> "Gson":
        """
        Creates a Gson instance based on the current configuration. This method is free of
        side-effects to this `GsonBuilder` instance and hence can be called multiple times.

        Returns
        - an instance of Gson configured with the options currently set in this builder
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.DragType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.event.inventory import *
from typing import Any, Callable, Iterable, Tuple


class DragType(Enum):
    """
    Represents the effect of a drag that will be applied to an Inventory in an
    InventoryDragEvent.
    """

    SINGLE = 0
    """
    One item from the cursor is placed in each selected slot.
    """
    EVEN = 1
    """
    The cursor is split evenly across all selected slots, not to exceed the
    Material's max stack size, with the remainder going to the cursor.
    """



"""
Python module generated from Java source file org.bukkit.inventory.SmithingTransformRecipe

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import NamespacedKey
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class SmithingTransformRecipe(SmithingRecipe):
    """
    Represents a smithing transform recipe.
    """

    def __init__(self, key: "NamespacedKey", result: "ItemStack", template: "RecipeChoice", base: "RecipeChoice", addition: "RecipeChoice"):
        """
        Create a smithing recipe to produce the specified result ItemStack.

        Arguments
        - key: The unique recipe key
        - result: The item you want the recipe to create.
        - template: The template item.
        - base: The base ingredient
        - addition: The addition ingredient
        """
        ...


    def getTemplate(self) -> "RecipeChoice":
        """
        Get the template recipe item.

        Returns
        - template choice
        """
        ...



from .ResourceBundle import ResourceBundle
from .Arrays import Arrays
from .Collections import Collections
from .IdentityHashMap import IdentityHashMap
from .Locale import Locale
from .Objects import Objects
from .UUID import UUID
from .Iterator import Iterator
from .NoSuchElementException import NoSuchElementException
from .ArrayDeque import ArrayDeque
from .Date import Date
from .AbstractList import AbstractList
from .RandomAccess import RandomAccess
from .AbstractSet import AbstractSet
from .Comparator import Comparator
from .ConcurrentModificationException import ConcurrentModificationException
from .EnumMap import EnumMap
from .EnumSet import EnumSet
from .LinkedHashSet import LinkedHashSet
from .Queue import Queue
from .SortedMap import SortedMap
from .SortedSet import SortedSet
from .Properties import Properties
from .TimeZone import TimeZone
from .BitSet import BitSet
from .Calendar import Calendar
from .Currency import Currency
from .Deque import Deque
from .GregorianCalendar import GregorianCalendar
from .StringTokenizer import StringTokenizer
from .Enumeration import Enumeration
from .Spliterator import Spliterator
from .Random import Random
from .Optional import Optional
from .NavigableSet import NavigableSet
from .ListIterator import ListIterator
from .NavigableMap import NavigableMap
from .OptionalDouble import OptionalDouble
from .OptionalInt import OptionalInt
from .OptionalLong import OptionalLong
from .PrimitiveIterator import PrimitiveIterator
from .PriorityQueue import PriorityQueue
from .AbstractCollection import AbstractCollection
from .AbstractQueue import AbstractQueue
from .AbstractSequentialList import AbstractSequentialList
from .ServiceConfigurationError import ServiceConfigurationError
from .WeakHashMap import WeakHashMap
from .IllegalFormatException import IllegalFormatException
from .EventObject import EventObject
from .EventListener import EventListener


"""
Python module generated from Java source file org.bukkit.entity.Villager

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import Lists
from java.util import Locale
from java.util import UUID
from org.bukkit import Bukkit
from org.bukkit import Keyed
from org.bukkit import Location
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit.entity import *
from org.bukkit.registry import RegistryAware
from org.bukkit.util import OldEnum
from typing import Any, Callable, Iterable, Tuple


class Villager(AbstractVillager):
    """
    Represents a villager NPC
    """

    def getProfession(self) -> "Profession":
        """
        Gets the current profession of this villager.

        Returns
        - Current profession.
        """
        ...


    def setProfession(self, profession: "Profession") -> None:
        """
        Sets the new profession of this villager.

        Arguments
        - profession: New profession.
        """
        ...


    def getVillagerType(self) -> "Type":
        """
        Gets the current type of this villager.

        Returns
        - Current type.
        """
        ...


    def setVillagerType(self, type: "Type") -> None:
        """
        Sets the new type of this villager.

        Arguments
        - type: New type.
        """
        ...


    def getVillagerLevel(self) -> int:
        """
        Gets the level of this villager.
        
        A villager with a level of 1 and no experience is liable to lose its
        profession.

        Returns
        - this villager's level
        """
        ...


    def setVillagerLevel(self, level: int) -> None:
        """
        Sets the level of this villager.
        
        A villager with a level of 1 and no experience is liable to lose its
        profession.

        Arguments
        - level: the new level

        Raises
        - IllegalArgumentException: if level not between [1, 5]
        """
        ...


    def getVillagerExperience(self) -> int:
        """
        Gets the trading experience of this villager.

        Returns
        - trading experience
        """
        ...


    def setVillagerExperience(self, experience: int) -> None:
        """
        Sets the trading experience of this villager.

        Arguments
        - experience: new experience

        Raises
        - IllegalArgumentException: if experience &lt; 0
        """
        ...


    def sleep(self, location: "Location") -> bool:
        """
        Attempts to make this villager sleep at the given location.
        
        The location must be in the current world and have a bed placed at the
        location. The villager will put its head on the specified block while
        sleeping.

        Arguments
        - location: the location of the bed

        Returns
        - whether the sleep was successful
        """
        ...


    def wakeup(self) -> None:
        """
        Causes this villager to wake up if he's currently sleeping.

        Raises
        - IllegalStateException: if not sleeping
        """
        ...


    def shakeHead(self) -> None:
        """
        Causes this villager to shake his head.
        """
        ...


    def zombify(self) -> "ZombieVillager":
        """
        Convert this Villager into a ZombieVillager as if it was killed by a
        Zombie.
        
        **Note:** this will fire a EntityTransformEvent

        Returns
        - the converted entity ZombieVillager or null if the
        conversion its cancelled
        """
        ...


    def getReputation(self, uuid: "UUID", reputationType: "ReputationType") -> int:
        """
        Gets the reputation of an entity for a given type.

        Arguments
        - uuid: the UUID of the entity whose reputation is being checked
        - reputationType: reputation type to be retrieved

        Returns
        - current reputation for the given reputation type
        """
        ...


    def getWeightedReputation(self, uuid: "UUID", reputationType: "ReputationType") -> int:
        """
        Gets the weighted reputation of an entity for a given type.
        
        The total reputation of an entity is a sum of its weighted
        reputations of each type, where the reputation is multiplied by weight
        assigned to its type.

        Arguments
        - uuid: the UUID of the entity whose reputation is being checked
        - reputationType: reputation type to be retrieved

        Returns
        - current reputation for the given reputation type

        See
        - ReputationType.getWeight()
        """
        ...


    def getReputation(self, uuid: "UUID") -> int:
        """
        Gets the reputation of an entity.

        Arguments
        - uuid: the UUID of the entity whose reputation is being checked

        Returns
        - current reputation for the given reputation type
        """
        ...


    def addReputation(self, uuid: "UUID", reputationType: "ReputationType", amount: int) -> None:
        """
        Add reputation of a given type towards a given entity.
        
        The final value will be clamped to the maximum value supported by the
        provided reputation type. If the final value is below the reputation
        discard threshold, gossip associated with this reputation type will be
        removed.
        
        Note: this will fire a
        org.bukkit.event.entity.VillagerReputationChangeEvent.

        Arguments
        - uuid: the UUID of the entity for whom the reputation is being
                    added
        - reputationType: reputation type to be modified
        - amount: amount of reputation to add
        """
        ...


    def addReputation(self, uuid: "UUID", reputationType: "ReputationType", amount: int, changeReason: "ReputationEvent") -> None:
        """
        Add reputation of a given type towards a given entity, with a specific
        change reason.
        
        The final value will be clamped to the maximum value supported by the
        provided reputation type. If the final value is below the reputation
        discard threshold, gossip associated with this reputation type will be
        removed.
        
        Note: this will fire a
        org.bukkit.event.entity.VillagerReputationChangeEvent.

        Arguments
        - uuid: the UUID of the entity for whom the reputation is being
                    added
        - reputationType: reputation type to be modified
        - amount: amount of reputation to add
        - changeReason: reputation change reason
        """
        ...


    def removeReputation(self, uuid: "UUID", reputationType: "ReputationType", amount: int) -> None:
        """
        Remove reputation of a given type towards a given entity.
        
        The final value will be clamped to the maximum value supported by the
        provided reputation type. If the final value is below the reputation
        discard threshold, gossip associated with this reputation type will be
        removed.
        
        Note: this will fire a
        org.bukkit.event.entity.VillagerReputationChangeEvent.

        Arguments
        - uuid: the UUID of the entity for whom the reputation is being
                    removed
        - reputationType: reputation type to be modified
        - amount: amount of reputation to remove
        """
        ...


    def removeReputation(self, uuid: "UUID", reputationType: "ReputationType", amount: int, changeReason: "ReputationEvent") -> None:
        """
        Remove reputation of a given type towards a given entity, with a
        specific change reason.
        
        The final value will be clamped to the maximum value supported by the
        provided reputation type. If the final value is below the reputation
        discard threshold, gossip associated with this reputation type will be
        removed.
        
        Note: this will fire a
        org.bukkit.event.entity.VillagerReputationChangeEvent.

        Arguments
        - uuid: the UUID of the entity for whom the reputation is being
                    removed
        - reputationType: reputation type to be modified
        - amount: amount of reputation to remove
        - changeReason: reputation change reason
        """
        ...


    def setReputation(self, uuid: "UUID", reputationType: "ReputationType", amount: int) -> None:
        """
        Set reputation of a given type towards a given entity.
        
        The final value will be clamped to the maximum value supported by the
        provided reputation type. If the final value is below the reputation
        discard threshold, gossip associated with this reputation type will be
        removed.
        
        Note: this will fire a
        org.bukkit.event.entity.VillagerReputationChangeEvent.

        Arguments
        - uuid: the UUID of the entity for whom the reputation is being
                    added
        - reputationType: reputation type to be modified
        - amount: amount of reputation to add
        """
        ...


    def setReputation(self, uuid: "UUID", reputationType: "ReputationType", amount: int, changeReason: "ReputationEvent") -> None:
        """
        Set reputation of a given type towards a given entity, with a specific
        change reason.
        
        The final value will be clamped to the maximum value supported by the
        provided reputation type. If the final value is below the reputation
        discard threshold, gossip associated with this reputation type will be
        removed.
        
        Note: this will fire a
        org.bukkit.event.entity.VillagerReputationChangeEvent.

        Arguments
        - uuid: the UUID of the entity for whom the reputation is being
                    added
        - reputationType: reputation type to be modified
        - amount: amount of reputation to add
        - changeReason: reputation change reason
        """
        ...


    def setGossipDecayTime(self, ticks: int) -> None:
        """
        Sets the reputation decay time for this villager.
        
        Defaults to **24000** (1 daylight cycle).

        Arguments
        - ticks: amount of ticks until the villager's reputation decays
        """
        ...


    def getGossipDecayTime(self) -> int:
        """
        Gets the reputation decay time for this villager.
        
        Defaults to **24000** (1 daylight cycle).

        Returns
        - amount of ticks until the villager's reputation decays
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockCookEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class BlockCookEvent(BlockEvent, Cancellable):
    """
    Called when an ItemStack is successfully cooked in a block.
    """

    def __init__(self, block: "Block", source: "ItemStack", result: "ItemStack"):
        ...


    def getSource(self) -> "ItemStack":
        """
        Gets the smelted ItemStack for this event

        Returns
        - smelting source ItemStack
        """
        ...


    def getResult(self) -> "ItemStack":
        """
        Gets the resultant ItemStack for this event

        Returns
        - smelting result ItemStack
        """
        ...


    def setResult(self, result: "ItemStack") -> None:
        """
        Sets the resultant ItemStack for this event

        Arguments
        - result: new result ItemStack
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.entity.Bat

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Bat(Ambient):
    """
    Represents a Bat
    """

    def isAwake(self) -> bool:
        """
        Checks the current waking state of this bat.
        
        This does not imply any persistence of state past the method call.

        Returns
        - True if the bat is awake or False if it is currently hanging
            from a block
        """
        ...


    def setAwake(self, state: bool) -> None:
        """
        This method modifies the current waking state of this bat.
        
        This does not prevent a bat from spontaneously awaking itself, or from
        reattaching itself to a block.

        Arguments
        - state: the new state
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.CraftingRecipe

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit import Keyed
from org.bukkit import Material
from org.bukkit import NamespacedKey
from org.bukkit.inventory import *
from org.bukkit.inventory.recipe import CraftingBookCategory
from typing import Any, Callable, Iterable, Tuple


class CraftingRecipe(Recipe, Keyed):
    """
    Represents a shaped or shapeless crafting recipe.
    """

    def getKey(self) -> "NamespacedKey":
        ...


    def getResult(self) -> "ItemStack":
        """
        Get the result of this recipe.

        Returns
        - The result stack.
        """
        ...


    def getGroup(self) -> str:
        """
        Get the group of this recipe. Recipes with the same group may be grouped
        together when displayed in the client.

        Returns
        - recipe group. An empty string denotes no group. May not be null.
        """
        ...


    def setGroup(self, group: str) -> None:
        """
        Set the group of this recipe. Recipes with the same group may be grouped
        together when displayed in the client.

        Arguments
        - group: recipe group. An empty string denotes no group. May not be
        null.
        """
        ...


    def getCategory(self) -> "CraftingBookCategory":
        """
        Gets the category which this recipe will appear in the recipe book under.
        
        Defaults to CraftingBookCategory.MISC if not set.

        Returns
        - recipe book category
        """
        ...


    def setCategory(self, category: "CraftingBookCategory") -> None:
        """
        Sets the category which this recipe will appear in the recipe book under.
        
        Defaults to CraftingBookCategory.MISC if not set.

        Arguments
        - category: recipe book category
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.Service

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotMock
from java.time import Duration
from java.util.concurrent import Executor
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from typing import Any, Callable, Iterable, Tuple


class Service:
    """
    An object with an operational state, plus asynchronous .startAsync() and .stopAsync() lifecycle methods to transition between states. Example services include
    webservers, RPC servers and timers.
    
    The normal lifecycle of a service is:
    
    
      - State.NEW NEW -&gt;
      - State.STARTING STARTING -&gt;
      - State.RUNNING RUNNING -&gt;
      - State.STOPPING STOPPING -&gt;
      - State.TERMINATED TERMINATED
    
    
    There are deviations from this if there are failures or if Service.stopAsync is called
    before the Service reaches the State.RUNNING RUNNING state. The set of legal
    transitions form a <a href="http://en.wikipedia.org/wiki/Directed_acyclic_graph">DAG</a>,
    therefore every method of the listener will be called at most once. N.B. The State.FAILED
    and State.TERMINATED states are terminal states, once a service enters either of these
    states it cannot ever leave them.
    
    Implementors of this interface are strongly encouraged to extend one of the abstract classes
    in this package which implement this interface and make the threading and state management
    easier.

    Author(s)
    - Luke Sandberg

    Since
    - 9.0 (in 1.0 as `com.google.common.base.Service`)
    """

    def startAsync(self) -> "Service":
        """
        If the service state is State.NEW, this initiates service startup and returns
        immediately. A stopped service may not be restarted.

        Returns
        - this

        Raises
        - IllegalStateException: if the service is not State.NEW

        Since
        - 15.0
        """
        ...


    def isRunning(self) -> bool:
        """
        Returns `True` if this service is State.RUNNING running.
        """
        ...


    def state(self) -> "State":
        """
        Returns the lifecycle state of the service.
        """
        ...


    def stopAsync(self) -> "Service":
        """
        If the service is State.STARTING starting or State.RUNNING running,
        this initiates service shutdown and returns immediately. If the service is State.NEW new, it is State.TERMINATED terminated without having been started nor
        stopped. If the service has already been stopped, this method returns immediately without
        taking action.

        Returns
        - this

        Since
        - 15.0
        """
        ...


    def awaitRunning(self) -> None:
        """
        Waits for the Service to reach the State.RUNNING running state.

        Raises
        - IllegalStateException: if the service reaches a state from which it is not possible to
            enter the State.RUNNING state. e.g. if the `state` is `State.TERMINATED` when this method is called then this will throw an IllegalStateException.

        Since
        - 15.0
        """
        ...


    def awaitRunning(self, timeout: "Duration") -> None:
        """
        Waits for the Service to reach the State.RUNNING running state for no more
        than the given time.

        Arguments
        - timeout: the maximum time to wait

        Raises
        - TimeoutException: if the service has not reached the given state within the deadline
        - IllegalStateException: if the service reaches a state from which it is not possible to
            enter the State.RUNNING RUNNING state. e.g. if the `state` is `State.TERMINATED` when this method is called then this will throw an IllegalStateException.

        Since
        - 28.0
        """
        ...


    def awaitRunning(self, timeout: int, unit: "TimeUnit") -> None:
        """
        Waits for the Service to reach the State.RUNNING running state for no more
        than the given time.

        Arguments
        - timeout: the maximum time to wait
        - unit: the time unit of the timeout argument

        Raises
        - TimeoutException: if the service has not reached the given state within the deadline
        - IllegalStateException: if the service reaches a state from which it is not possible to
            enter the State.RUNNING RUNNING state. e.g. if the `state` is `State.TERMINATED` when this method is called then this will throw an IllegalStateException.

        Since
        - 15.0
        """
        ...


    def awaitTerminated(self) -> None:
        """
        Waits for the Service to reach the State.TERMINATED terminated state.

        Raises
        - IllegalStateException: if the service State.FAILED fails.

        Since
        - 15.0
        """
        ...


    def awaitTerminated(self, timeout: "Duration") -> None:
        """
        Waits for the Service to reach a terminal state (either Service.State.TERMINATED
        terminated or Service.State.FAILED failed) for no more than the given time.

        Arguments
        - timeout: the maximum time to wait

        Raises
        - TimeoutException: if the service has not reached the given state within the deadline
        - IllegalStateException: if the service State.FAILED fails.

        Since
        - 28.0
        """
        ...


    def awaitTerminated(self, timeout: int, unit: "TimeUnit") -> None:
        """
        Waits for the Service to reach a terminal state (either Service.State.TERMINATED
        terminated or Service.State.FAILED failed) for no more than the given time.

        Arguments
        - timeout: the maximum time to wait
        - unit: the time unit of the timeout argument

        Raises
        - TimeoutException: if the service has not reached the given state within the deadline
        - IllegalStateException: if the service State.FAILED fails.

        Since
        - 15.0
        """
        ...


    def failureCause(self) -> "Throwable":
        """
        Returns the Throwable that caused this service to fail.

        Raises
        - IllegalStateException: if this service's state isn't State.FAILED FAILED.

        Since
        - 14.0
        """
        ...


    def addListener(self, listener: "Listener", executor: "Executor") -> None:
        """
        Registers a Listener to be Executor.execute executed on the given
        executor. The listener will have the corresponding transition method called whenever the
        service changes state. The listener will not have previous state changes replayed, so it is
        suggested that listeners are added before the service starts.
        
        `addListener` guarantees execution ordering across calls to a given listener but not
        across calls to multiple listeners. Specifically, a given listener will have its callbacks
        invoked in the same order as the underlying service enters those states. Additionally, at most
        one of the listener's callbacks will execute at once. However, multiple listeners' callbacks
        may execute concurrently, and listeners may execute in an order different from the one in which
        they were registered.
        
        RuntimeExceptions thrown by a listener will be caught and logged. Any exception thrown
        during `Executor.execute` (e.g., a `RejectedExecutionException`) will be caught and
        logged.

        Arguments
        - listener: the listener to run when the service changes state is complete
        - executor: the executor in which the listeners callback methods will be run. For fast,
            lightweight listeners that would be safe to execute in any thread, consider MoreExecutors.directExecutor.

        Since
        - 13.0
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.AtomicDoubleArray

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.primitives import ImmutableLongArray
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import Serializable
from java.util.concurrent.atomic import AtomicLongArray
from java.util.function import DoubleBinaryOperator
from java.util.function import DoubleUnaryOperator
from typing import Any, Callable, Iterable, Tuple


class AtomicDoubleArray(Serializable):
    """
    A `double` array in which elements may be updated atomically. See the java.util.concurrent.atomic package specification for description of the properties of atomic
    variables.
    
    <a id="bitEquals"></a>This class compares primitive `double` values in methods such as
    .compareAndSet by comparing their bitwise representation using Double.doubleToRawLongBits, which differs from both the primitive double `==` operator and
    from Double.equals, as if implemented by:
    
    ````static boolean bitEquals(double x, double y) {
      long xBits = Double.doubleToRawLongBits(x);
      long yBits = Double.doubleToRawLongBits(y);
      return xBits == yBits;`
    }```

    Author(s)
    - Martin Buchholz

    Since
    - 11.0
    """

    def __init__(self, length: int):
        """
        Creates a new `AtomicDoubleArray` of the given length, with all elements initially zero.

        Arguments
        - length: the length of the array
        """
        ...


    def __init__(self, array: list[float]):
        """
        Creates a new `AtomicDoubleArray` with the same length as, and all elements copied from,
        the given array.

        Arguments
        - array: the array to copy elements from

        Raises
        - NullPointerException: if array is null
        """
        ...


    def length(self) -> int:
        """
        Returns the length of the array.

        Returns
        - the length of the array
        """
        ...


    def get(self, i: int) -> float:
        """
        Gets the current value at position `i`.

        Arguments
        - i: the index

        Returns
        - the current value
        """
        ...


    def set(self, i: int, newValue: float) -> None:
        """
        Atomically sets the element at position `i` to the given value.

        Arguments
        - i: the index
        - newValue: the new value
        """
        ...


    def lazySet(self, i: int, newValue: float) -> None:
        """
        Eventually sets the element at position `i` to the given value.

        Arguments
        - i: the index
        - newValue: the new value
        """
        ...


    def getAndSet(self, i: int, newValue: float) -> float:
        """
        Atomically sets the element at position `i` to the given value and returns the old value.

        Arguments
        - i: the index
        - newValue: the new value

        Returns
        - the previous value
        """
        ...


    def compareAndSet(self, i: int, expect: float, update: float) -> bool:
        """
        Atomically sets the element at position `i` to the given updated value if the current
        value is <a href="#bitEquals">bitwise equal</a> to the expected value.

        Arguments
        - i: the index
        - expect: the expected value
        - update: the new value

        Returns
        - True if successful. False return indicates that the actual value was not equal to the
            expected value.
        """
        ...


    def weakCompareAndSet(self, i: int, expect: float, update: float) -> bool:
        """
        Atomically sets the element at position `i` to the given updated value if the current
        value is <a href="#bitEquals">bitwise equal</a> to the expected value.
        
        May <a
        href="http://download.oracle.com/javase/7/docs/api/java/util/concurrent/atomic/package-summary.html#Spurious">
        fail spuriously</a> and does not provide ordering guarantees, so is only rarely an appropriate
        alternative to `compareAndSet`.

        Arguments
        - i: the index
        - expect: the expected value
        - update: the new value

        Returns
        - True if successful
        """
        ...


    def getAndAdd(self, i: int, delta: float) -> float:
        """
        Atomically adds the given value to the element at index `i`.

        Arguments
        - i: the index
        - delta: the value to add

        Returns
        - the previous value
        """
        ...


    def addAndGet(self, i: int, delta: float) -> float:
        """
        Atomically adds the given value to the element at index `i`.

        Arguments
        - i: the index
        - delta: the value to add

        Returns
        - the updated value
        """
        ...


    def getAndAccumulate(self, i: int, x: float, accumulatorFunction: "DoubleBinaryOperator") -> float:
        """
        Atomically updates the element at index `i` with the results of applying the given
        function to the current and given values.

        Arguments
        - i: the index to update
        - x: the update value
        - accumulatorFunction: the accumulator function

        Returns
        - the previous value

        Since
        - 31.1
        """
        ...


    def accumulateAndGet(self, i: int, x: float, accumulatorFunction: "DoubleBinaryOperator") -> float:
        """
        Atomically updates the element at index `i` with the results of applying the given
        function to the current and given values.

        Arguments
        - i: the index to update
        - x: the update value
        - accumulatorFunction: the accumulator function

        Returns
        - the updated value

        Since
        - 31.1
        """
        ...


    def getAndUpdate(self, i: int, updaterFunction: "DoubleUnaryOperator") -> float:
        """
        Atomically updates the element at index `i` with the results of applying the given
        function to the current value.

        Arguments
        - i: the index to update
        - updaterFunction: the update function

        Returns
        - the previous value

        Since
        - 31.1
        """
        ...


    def updateAndGet(self, i: int, updaterFunction: "DoubleUnaryOperator") -> float:
        """
        Atomically updates the element at index `i` with the results of applying the given
        function to the current value.

        Arguments
        - i: the index to update
        - updaterFunction: the update function

        Returns
        - the updated value

        Since
        - 31.1
        """
        ...


    def toString(self) -> str:
        """
        Returns the String representation of the current values of array.

        Returns
        - the String representation of the current values of array
        """
        ...



"""
Python module generated from Java source file org.bukkit.map.MapCursor

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import Lists
from java.util import Locale
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit.map import *
from org.bukkit.registry import RegistryAware
from org.bukkit.util import OldEnum
from typing import Any, Callable, Iterable, Tuple


class MapCursor:
    """
    Represents a cursor on a map.
    """

    def __init__(self, x: int, y: int, direction: int, type: int, visible: bool):
        """
        Initialize the map cursor.

        Arguments
        - x: The x coordinate, from -128 to 127.
        - y: The y coordinate, from -128 to 127.
        - direction: The facing of the cursor, from 0 to 15.
        - type: The type (color/style) of the map cursor.
        - visible: Whether the cursor is visible by default.

        Deprecated
        - Magic value
        """
        ...


    def __init__(self, x: int, y: int, direction: int, type: "Type", visible: bool):
        """
        Initialize the map cursor.

        Arguments
        - x: The x coordinate, from -128 to 127.
        - y: The y coordinate, from -128 to 127.
        - direction: The facing of the cursor, from 0 to 15.
        - type: The type (color/style) of the map cursor.
        - visible: Whether the cursor is visible by default.
        """
        ...


    def __init__(self, x: int, y: int, direction: int, type: int, visible: bool, caption: str):
        """
        Initialize the map cursor.

        Arguments
        - x: The x coordinate, from -128 to 127.
        - y: The y coordinate, from -128 to 127.
        - direction: The facing of the cursor, from 0 to 15.
        - type: The type (color/style) of the map cursor.
        - visible: Whether the cursor is visible by default.
        - caption: cursor caption

        Deprecated
        - Magic value, use .MapCursor(byte, byte, byte, Type, boolean, String)
        """
        ...


    def __init__(self, x: int, y: int, direction: int, type: "Type", visible: bool, caption: str):
        """
        Initialize the map cursor.

        Arguments
        - x: The x coordinate, from -128 to 127.
        - y: The y coordinate, from -128 to 127.
        - direction: The facing of the cursor, from 0 to 15.
        - type: The type (color/style) of the map cursor.
        - visible: Whether the cursor is visible by default.
        - caption: cursor caption
        """
        ...


    def getX(self) -> int:
        """
        Get the X position of this cursor.

        Returns
        - The X coordinate.
        """
        ...


    def getY(self) -> int:
        """
        Get the Y position of this cursor.

        Returns
        - The Y coordinate.
        """
        ...


    def getDirection(self) -> int:
        """
        Get the direction of this cursor.

        Returns
        - The facing of the cursor, from 0 to 15.
        """
        ...


    def getType(self) -> "Type":
        """
        Get the type of this cursor.

        Returns
        - The type (color/style) of the map cursor.
        """
        ...


    def getRawType(self) -> int:
        """
        Get the type of this cursor.

        Returns
        - The type (color/style) of the map cursor.

        Deprecated
        - Magic value
        """
        ...


    def isVisible(self) -> bool:
        """
        Get the visibility status of this cursor.

        Returns
        - True if visible, False otherwise.
        """
        ...


    def setX(self, x: int) -> None:
        """
        Set the X position of this cursor.

        Arguments
        - x: The X coordinate.
        """
        ...


    def setY(self, y: int) -> None:
        """
        Set the Y position of this cursor.

        Arguments
        - y: The Y coordinate.
        """
        ...


    def setDirection(self, direction: int) -> None:
        """
        Set the direction of this cursor.

        Arguments
        - direction: The facing of the cursor, from 0 to 15.
        """
        ...


    def setType(self, type: "Type") -> None:
        """
        Set the type of this cursor.

        Arguments
        - type: The type (color/style) of the map cursor.
        """
        ...


    def setRawType(self, type: int) -> None:
        """
        Set the type of this cursor.

        Arguments
        - type: The type (color/style) of the map cursor.

        Deprecated
        - Magic value
        """
        ...


    def setVisible(self, visible: bool) -> None:
        """
        Set the visibility status of this cursor.

        Arguments
        - visible: True if visible.
        """
        ...


    def getCaption(self) -> str:
        """
        Gets the caption on this cursor.

        Returns
        - caption
        """
        ...


    def setCaption(self, caption: str) -> None:
        """
        Sets the caption on this cursor.

        Arguments
        - caption: new caption
        """
        ...


    class Type(OldEnum, Keyed, RegistryAware):
        """
        Represents the standard types of map cursors. More may be made
        available by resource packs - the value is used by the client as an
        index in the file './assets/minecraft/textures/map/map_icons.png' from minecraft.jar or from a
        resource pack.
        """

        PLAYER = getType("player")
        FRAME = getType("frame")
        RED_MARKER = getType("red_marker")
        BLUE_MARKER = getType("blue_marker")
        TARGET_X = getType("target_x")
        TARGET_POINT = getType("target_point")
        PLAYER_OFF_MAP = getType("player_off_map")
        PLAYER_OFF_LIMITS = getType("player_off_limits")
        MANSION = getType("mansion")
        MONUMENT = getType("monument")
        BANNER_WHITE = getType("banner_white")
        BANNER_ORANGE = getType("banner_orange")
        BANNER_MAGENTA = getType("banner_magenta")
        BANNER_LIGHT_BLUE = getType("banner_light_blue")
        BANNER_YELLOW = getType("banner_yellow")
        BANNER_LIME = getType("banner_lime")
        BANNER_PINK = getType("banner_pink")
        BANNER_GRAY = getType("banner_gray")
        BANNER_LIGHT_GRAY = getType("banner_light_gray")
        BANNER_CYAN = getType("banner_cyan")
        BANNER_PURPLE = getType("banner_purple")
        BANNER_BLUE = getType("banner_blue")
        BANNER_BROWN = getType("banner_brown")
        BANNER_GREEN = getType("banner_green")
        BANNER_RED = getType("banner_red")
        BANNER_BLACK = getType("banner_black")
        RED_X = getType("red_x")
        VILLAGE_DESERT = getType("village_desert")
        VILLAGE_PLAINS = getType("village_plains")
        VILLAGE_SAVANNA = getType("village_savanna")
        VILLAGE_SNOWY = getType("village_snowy")
        VILLAGE_TAIGA = getType("village_taiga")
        JUNGLE_TEMPLE = getType("jungle_temple")
        SWAMP_HUT = getType("swamp_hut")
        TRIAL_CHAMBERS = getType("trial_chambers")


        @staticmethod
        def getType(key: str) -> "Type":
            ...


        def getKey(self) -> "NamespacedKey":
            """
            See
            - .isRegistered()

            Deprecated
            - A key might not always be present, use .getKeyOrThrow() instead.
            """
            ...


        def getValue(self) -> int:
            """
            Gets the internal value of the cursor.

            Returns
            - the value

            Deprecated
            - Magic value
            """
            ...


        @staticmethod
        def byValue(value: int) -> "Type":
            """
            Get a cursor by its internal value.

            Arguments
            - value: the value

            Returns
            - the matching type

            Deprecated
            - Magic value
            """
            ...


        @staticmethod
        def valueOf(name: str) -> "Type":
            """
            Arguments
            - name: of the type.

            Returns
            - the type with the given name.

            Deprecated
            - only for backwards compatibility, use Registry.get(NamespacedKey) instead.
            """
            ...


        @staticmethod
        def values() -> list["Type"]:
            """
            Returns
            - an array of all known map cursor types.

            Deprecated
            - use Registry.iterator().
            """
            ...



"""
Python module generated from Java source file org.bukkit.util.BlockTransformer

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import BlockState
from org.bukkit.block.data import BlockData
from org.bukkit.generator import LimitedRegion
from org.bukkit.util import *
from typing import Any, Callable, Iterable, Tuple


class BlockTransformer:
    """
    A BlockTransformer is used to modify blocks that are placed by structure.
    """

    def transform(self, region: "LimitedRegion", x: int, y: int, z: int, current: "BlockState", state: "TransformationState") -> "BlockState":
        """
        Transforms a block in a structure.
        
        NOTE: The usage of BlockData.createBlockState() can provide even
        more flexibility to return the exact block state you might want to
        return.

        Arguments
        - region: the accessible region
        - x: the x position of the block
        - y: the y position of the block
        - z: the z position of the block
        - current: the state of the block that should be placed
        - state: the state of this transformation.

        Returns
        - the new block state
        """
        ...


    class TransformationState:
        """
        The TransformationState allows access to the original block state and the
        block state of the block that was at the location of the transformation
        in the world before the transformation started.
        """

        def getOriginal(self) -> "BlockState":
            """
            Creates a clone of the original block state that a structure wanted
            to place and caches it for the current transformer.

            Returns
            - a clone of the original block state.
            """
            ...


        def getWorld(self) -> "BlockState":
            """
            Creates a clone of the block state that was at the location of the
            currently modified block at the start of the transformation process
            and caches it for the current transformer.

            Returns
            - a clone of the world block state.
            """
            ...



"""
Python module generated from Java source file com.google.common.util.concurrent.MoreExecutors

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import Supplier
from com.google.common.base import Throwables
from com.google.common.collect import Lists
from com.google.common.collect import Queues
from com.google.common.util.concurrent import *
from com.google.common.util.concurrent.ForwardingListenableFuture import SimpleForwardingListenableFuture
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.lang.reflect import InvocationTargetException
from java.time import Duration
from java.util import Iterator
from java.util.concurrent import BlockingQueue
from java.util.concurrent import Callable
from java.util.concurrent import Delayed
from java.util.concurrent import ExecutionException
from java.util.concurrent import Executor
from java.util.concurrent import ExecutorService
from java.util.concurrent import Executors
from java.util.concurrent import Future
from java.util.concurrent import RejectedExecutionException
from java.util.concurrent import ScheduledExecutorService
from java.util.concurrent import ScheduledFuture
from java.util.concurrent import ScheduledThreadPoolExecutor
from java.util.concurrent import ThreadFactory
from java.util.concurrent import ThreadPoolExecutor
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class MoreExecutors:
    """
    Factory and utility methods for java.util.concurrent.Executor, ExecutorService,
    and java.util.concurrent.ThreadFactory.

    Author(s)
    - Justin Mahoney

    Since
    - 3.0
    """

    @staticmethod
    def getExitingExecutorService(executor: "ThreadPoolExecutor", terminationTimeout: "Duration") -> "ExecutorService":
        """
        Converts the given ThreadPoolExecutor into an ExecutorService that exits when the application
        is complete. It does so by using daemon threads and adding a shutdown hook to wait for their
        completion.
        
        This is mainly for fixed thread pools. See Executors.newFixedThreadPool(int).

        Arguments
        - executor: the executor to modify to make sure it exits when the application is finished
        - terminationTimeout: how long to wait for the executor to finish before terminating the
            JVM

        Returns
        - an unmodifiable version of the input which will not hang the JVM

        Since
        - 28.0
        """
        ...


    @staticmethod
    def getExitingExecutorService(executor: "ThreadPoolExecutor", terminationTimeout: int, timeUnit: "TimeUnit") -> "ExecutorService":
        """
        Converts the given ThreadPoolExecutor into an ExecutorService that exits when the application
        is complete. It does so by using daemon threads and adding a shutdown hook to wait for their
        completion.
        
        This is mainly for fixed thread pools. See Executors.newFixedThreadPool(int).

        Arguments
        - executor: the executor to modify to make sure it exits when the application is finished
        - terminationTimeout: how long to wait for the executor to finish before terminating the
            JVM
        - timeUnit: unit of time for the time parameter

        Returns
        - an unmodifiable version of the input which will not hang the JVM
        """
        ...


    @staticmethod
    def getExitingExecutorService(executor: "ThreadPoolExecutor") -> "ExecutorService":
        """
        Converts the given ThreadPoolExecutor into an ExecutorService that exits when the application
        is complete. It does so by using daemon threads and adding a shutdown hook to wait for their
        completion.
        
        This method waits 120 seconds before continuing with JVM termination, even if the executor
        has not finished its work.
        
        This is mainly for fixed thread pools. See Executors.newFixedThreadPool(int).

        Arguments
        - executor: the executor to modify to make sure it exits when the application is finished

        Returns
        - an unmodifiable version of the input which will not hang the JVM
        """
        ...


    @staticmethod
    def getExitingScheduledExecutorService(executor: "ScheduledThreadPoolExecutor", terminationTimeout: "Duration") -> "ScheduledExecutorService":
        """
        Converts the given ScheduledThreadPoolExecutor into a ScheduledExecutorService that exits when
        the application is complete. It does so by using daemon threads and adding a shutdown hook to
        wait for their completion.
        
        This is mainly for fixed thread pools. See Executors.newScheduledThreadPool(int).

        Arguments
        - executor: the executor to modify to make sure it exits when the application is finished
        - terminationTimeout: how long to wait for the executor to finish before terminating the
            JVM

        Returns
        - an unmodifiable version of the input which will not hang the JVM

        Since
        - 28.0
        """
        ...


    @staticmethod
    def getExitingScheduledExecutorService(executor: "ScheduledThreadPoolExecutor", terminationTimeout: int, timeUnit: "TimeUnit") -> "ScheduledExecutorService":
        """
        Converts the given ScheduledThreadPoolExecutor into a ScheduledExecutorService that exits when
        the application is complete. It does so by using daemon threads and adding a shutdown hook to
        wait for their completion.
        
        This is mainly for fixed thread pools. See Executors.newScheduledThreadPool(int).

        Arguments
        - executor: the executor to modify to make sure it exits when the application is finished
        - terminationTimeout: how long to wait for the executor to finish before terminating the
            JVM
        - timeUnit: unit of time for the time parameter

        Returns
        - an unmodifiable version of the input which will not hang the JVM
        """
        ...


    @staticmethod
    def getExitingScheduledExecutorService(executor: "ScheduledThreadPoolExecutor") -> "ScheduledExecutorService":
        """
        Converts the given ScheduledThreadPoolExecutor into a ScheduledExecutorService that exits when
        the application is complete. It does so by using daemon threads and adding a shutdown hook to
        wait for their completion.
        
        This method waits 120 seconds before continuing with JVM termination, even if the executor
        has not finished its work.
        
        This is mainly for fixed thread pools. See Executors.newScheduledThreadPool(int).

        Arguments
        - executor: the executor to modify to make sure it exits when the application is finished

        Returns
        - an unmodifiable version of the input which will not hang the JVM
        """
        ...


    @staticmethod
    def addDelayedShutdownHook(service: "ExecutorService", terminationTimeout: "Duration") -> None:
        """
        Add a shutdown hook to wait for thread completion in the given ExecutorService service.
        This is useful if the given service uses daemon threads, and we want to keep the JVM from
        exiting immediately on shutdown, instead giving these daemon threads a chance to terminate
        normally.

        Arguments
        - service: ExecutorService which uses daemon threads
        - terminationTimeout: how long to wait for the executor to finish before terminating the
            JVM

        Since
        - 28.0
        """
        ...


    @staticmethod
    def addDelayedShutdownHook(service: "ExecutorService", terminationTimeout: int, timeUnit: "TimeUnit") -> None:
        """
        Add a shutdown hook to wait for thread completion in the given ExecutorService service.
        This is useful if the given service uses daemon threads, and we want to keep the JVM from
        exiting immediately on shutdown, instead giving these daemon threads a chance to terminate
        normally.

        Arguments
        - service: ExecutorService which uses daemon threads
        - terminationTimeout: how long to wait for the executor to finish before terminating the
            JVM
        - timeUnit: unit of time for the time parameter
        """
        ...


    @staticmethod
    def newDirectExecutorService() -> "ListeningExecutorService":
        """
        Creates an executor service that runs each task in the thread that invokes `execute/submit`, as in `ThreadPoolExecutor.CallerRunsPolicy`. This applies both to
        individually submitted tasks and to collections of tasks submitted via `invokeAll` or
        `invokeAny`. In the latter case, tasks will run serially on the calling thread. Tasks are
        run to completion before a `Future` is returned to the caller (unless the executor has
        been shutdown).
        
        Although all tasks are immediately executed in the thread that submitted the task, this
        `ExecutorService` imposes a small locking overhead on each task submission in order to
        implement shutdown and termination behavior.
        
        The implementation deviates from the `ExecutorService` specification with regards to
        the `shutdownNow` method. First, "best-effort" with regards to canceling running tasks is
        implemented as "no-effort". No interrupts or other attempts are made to stop threads executing
        tasks. Second, the returned list will always be empty, as any submitted task is considered to
        have started execution. This applies also to tasks given to `invokeAll` or `invokeAny` which are pending serial execution, even the subset of the tasks that have not yet
        started execution. It is unclear from the `ExecutorService` specification if these should
        be included, and it's much easier to implement the interpretation that they not be. Finally, a
        call to `shutdown` or `shutdownNow` may result in concurrent calls to `invokeAll/invokeAny` throwing RejectedExecutionException, although a subset of the tasks may
        already have been executed.

        Since
        - 18.0 (present as MoreExecutors.sameThreadExecutor() since 10.0)
        """
        ...


    @staticmethod
    def directExecutor() -> "Executor":
        """
        Returns an Executor that runs each task in the thread that invokes Executor.execute execute, as in `ThreadPoolExecutor.CallerRunsPolicy`.
        
        This executor is appropriate for tasks that are lightweight and not deeply chained.
        Inappropriate `directExecutor` usage can cause problems, and these problems can be
        difficult to reproduce because they depend on timing. For example:
        
        
          - When a `ListenableFuture` listener is registered to run under `directExecutor`, the listener can execute in any of three possible threads:
              <ol>
                - When a thread attaches a listener to a `ListenableFuture` that's already
                    complete, the listener runs immediately in that thread.
                - When a thread attaches a listener to a `ListenableFuture` that's
                    *in*complete and the `ListenableFuture` later completes normally, the
                    listener runs in the thread that completes the `ListenableFuture`.
                - When a listener is attached to a `ListenableFuture` and the `ListenableFuture` gets cancelled, the listener runs immediately in the thread that
                    cancelled the `Future`.
              </ol>
              Given all these possibilities, it is frequently possible for listeners to execute in UI
              threads, RPC network threads, or other latency-sensitive threads. In those cases, slow
              listeners can harm responsiveness, slow the system as a whole, or worse. (See also the
              note about locking below.)
          - If many tasks will be triggered by the same event, one heavyweight task may delay other
              tasks -- even tasks that are not themselves `directExecutor` tasks.
          - If many such tasks are chained together (such as with `future.transform(...).transform(...).transform(...)....`), they may overflow the stack.
              (In simple cases, callers can avoid this by registering all tasks with the same MoreExecutors.newSequentialExecutor wrapper around `directExecutor()`. More
              complex cases may require using thread pools or making deeper changes.)
          - If an exception propagates out of a `Runnable`, it is not necessarily seen by any
              `UncaughtExceptionHandler` for the thread. For example, if the callback passed to
              Futures.addCallback throws an exception, that exception will be typically be
              logged by the ListenableFuture implementation, even if the thread is configured
              to do something different. In other cases, no code will catch the exception, and it may
              terminate whichever thread happens to trigger the execution.
        
        
        A specific warning about locking: Code that executes user-supplied tasks, such as `ListenableFuture` listeners, should take care not to do so while holding a lock. Additionally,
        as a further line of defense, prefer not to perform any locking inside a task that will be run
        under `directExecutor`: Not only might the wait for a lock be long, but if the running
        thread was holding a lock, the listener may deadlock or break lock isolation.
        
        This instance is equivalent to:
        
        ````final class DirectExecutor implements Executor {
          public void execute(Runnable r) {
            r.run();`
        }
        }```
        
        This should be preferred to .newDirectExecutorService() because implementing the
        ExecutorService subinterface necessitates significant performance overhead.

        Since
        - 18.0
        """
        ...


    @staticmethod
    def newSequentialExecutor(delegate: "Executor") -> "Executor":
        """
        Returns an Executor that runs each task executed sequentially, such that no two tasks
        are running concurrently.
        
        Executor.execute executed tasks have a happens-before order as defined in the
        Java Language Specification. Tasks execute with the same happens-before order that the function
        calls to Executor.execute `execute()` that submitted those tasks had.
        
        The executor uses `delegate` in order to Executor.execute execute each task in
        turn, and does not create any threads of its own.
        
        After execution begins on a thread from the `delegate` Executor, tasks are
        polled and executed from a task queue until there are no more tasks. The thread will not be
        released until there are no more tasks to run.
        
        If a task is submitted while a thread is executing tasks from the task queue, the thread
        will not be released until that submitted task is also complete.
        
        If a task is Thread.interrupt interrupted while a task is running:
        
        <ol>
          - execution will not stop until the task queue is empty.
          - tasks will begin execution with the thread marked as not interrupted - any interruption
              applies only to the task that was running at the point of interruption.
          - if the thread was interrupted before the SequentialExecutor's worker begins execution,
              the interrupt will be restored to the thread after it completes so that its `delegate` Executor may process the interrupt.
          - subtasks are run with the thread uninterrupted and interrupts received during execution
              of a task are ignored.
        </ol>
        
        `RuntimeException`s thrown by tasks are simply logged and the executor keeps trucking.
        If an `Error` is thrown, the error will propagate and execution will stop until the next
        time a task is submitted.
        
        When an `Error` is thrown by an executed task, previously submitted tasks may never
        run. An attempt will be made to restart execution on the next call to `execute`. If the
        `delegate` has begun to reject execution, the previously submitted tasks may never run,
        despite not throwing a RejectedExecutionException synchronously with the call to `execute`. If this behaviour is problematic, use an Executor with a single thread (e.g. Executors.newSingleThreadExecutor).

        Since
        - 23.3 (since 23.1 as `sequentialExecutor`)
        """
        ...


    @staticmethod
    def listeningDecorator(delegate: "ExecutorService") -> "ListeningExecutorService":
        """
        Creates an ExecutorService whose `submit` and `invokeAll` methods submit
        ListenableFutureTask instances to the given delegate executor. Those methods, as well
        as `execute` and `invokeAny`, are implemented in terms of calls to `delegate.execute`. All other methods are forwarded unchanged to the delegate. This implies that
        the returned `ListeningExecutorService` never calls the delegate's `submit`, `invokeAll`, and `invokeAny` methods, so any special handling of tasks must be implemented
        in the delegate's `execute` method or by wrapping the returned `ListeningExecutorService`.
        
        If the delegate executor was already an instance of `ListeningExecutorService`, it is
        returned untouched, and the rest of this documentation does not apply.

        Since
        - 10.0
        """
        ...


    @staticmethod
    def listeningDecorator(delegate: "ScheduledExecutorService") -> "ListeningScheduledExecutorService":
        """
        Creates a ScheduledExecutorService whose `submit` and `invokeAll` methods
        submit ListenableFutureTask instances to the given delegate executor. Those methods, as
        well as `execute` and `invokeAny`, are implemented in terms of calls to `delegate.execute`. All other methods are forwarded unchanged to the delegate. This implies that
        the returned `ListeningScheduledExecutorService` never calls the delegate's `submit`, `invokeAll`, and `invokeAny` methods, so any special handling of tasks
        must be implemented in the delegate's `execute` method or by wrapping the returned `ListeningScheduledExecutorService`.
        
        If the delegate executor was already an instance of `ListeningScheduledExecutorService`, it is returned untouched, and the rest of this
        documentation does not apply.

        Since
        - 10.0
        """
        ...


    @staticmethod
    def platformThreadFactory() -> "ThreadFactory":
        """
        Returns a default thread factory used to create new threads.
        
        When running on AppEngine with access to <a
        href="https://cloud.google.com/appengine/docs/standard/java/javadoc/">AppEngine legacy
        APIs</a>, this method returns `ThreadManager.currentRequestThreadFactory()`. Otherwise,
        it returns Executors.defaultThreadFactory().

        Since
        - 14.0
        """
        ...


    @staticmethod
    def shutdownAndAwaitTermination(service: "ExecutorService", timeout: "Duration") -> bool:
        """
        Shuts down the given executor service gradually, first disabling new submissions and later, if
        necessary, cancelling remaining tasks.
        
        The method takes the following steps:
        
        <ol>
          - calls ExecutorService.shutdown(), disabling acceptance of new submitted tasks.
          - awaits executor service termination for half of the specified timeout.
          - if the timeout expires, it calls ExecutorService.shutdownNow(), cancelling
              pending tasks and interrupting running tasks.
          - awaits executor service termination for the other half of the specified timeout.
        </ol>
        
        If, at any step of the process, the calling thread is interrupted, the method calls ExecutorService.shutdownNow() and returns.

        Arguments
        - service: the `ExecutorService` to shut down
        - timeout: the maximum time to wait for the `ExecutorService` to terminate

        Returns
        - `True` if the `ExecutorService` was terminated successfully, `False`
            if the call timed out or was interrupted

        Since
        - 28.0
        """
        ...


    @staticmethod
    def shutdownAndAwaitTermination(service: "ExecutorService", timeout: int, unit: "TimeUnit") -> bool:
        """
        Shuts down the given executor service gradually, first disabling new submissions and later, if
        necessary, cancelling remaining tasks.
        
        The method takes the following steps:
        
        <ol>
          - calls ExecutorService.shutdown(), disabling acceptance of new submitted tasks.
          - awaits executor service termination for half of the specified timeout.
          - if the timeout expires, it calls ExecutorService.shutdownNow(), cancelling
              pending tasks and interrupting running tasks.
          - awaits executor service termination for the other half of the specified timeout.
        </ol>
        
        If, at any step of the process, the calling thread is interrupted, the method calls ExecutorService.shutdownNow() and returns.

        Arguments
        - service: the `ExecutorService` to shut down
        - timeout: the maximum time to wait for the `ExecutorService` to terminate
        - unit: the time unit of the timeout argument

        Returns
        - `True` if the `ExecutorService` was terminated successfully, `False`
            if the call timed out or was interrupted

        Since
        - 17.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.conversations.StringPrompt

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.conversations import *
from typing import Any, Callable, Iterable, Tuple


class StringPrompt(Prompt):
    """
    StringPrompt is the base class for any prompt that accepts an arbitrary
    string from the user.
    """

    def blocksForInput(self, context: "ConversationContext") -> bool:
        """
        Ensures that the prompt waits for the user to provide input.

        Arguments
        - context: Context information about the conversation.

        Returns
        - True.
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.TypeDescription

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Collections
from java.util import LinkedHashSet
from org.yaml.snakeyaml import *
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.internal import Logger
from org.yaml.snakeyaml.introspector import BeanAccess
from org.yaml.snakeyaml.introspector import Property
from org.yaml.snakeyaml.introspector import PropertySubstitute
from org.yaml.snakeyaml.introspector import PropertyUtils
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.nodes import Tag
from typing import Any, Callable, Iterable, Tuple


class TypeDescription:
    """
    Provides additional runtime information necessary to create a custom Java instance.
    
    In general this class is thread-safe and can be used as a singleton, the only exception being the
    PropertyUtils field. A singleton PropertyUtils should be constructed and shared between all YAML
    Constructors used if a singleton TypeDescription is used, since Constructor sets its
    propertyUtils to the TypeDescription that is passed to it, hence you may end up in a situation
    when propertyUtils in TypeDescription is from different Constructor.
    """

    def __init__(self, clazz: type["Object"], tag: "Tag"):
        ...


    def __init__(self, clazz: type["Object"], tag: "Tag", impl: type[Any]):
        ...


    def __init__(self, clazz: type["Object"], tag: str):
        ...


    def __init__(self, clazz: type["Object"]):
        ...


    def __init__(self, clazz: type["Object"], impl: type[Any]):
        ...


    def getTag(self) -> "Tag":
        """
        Get tag which shall be used to load or dump the type (class).

        Returns
        - tag to be used. It may be a tag for Language-Independent Types
                (http://www.yaml.org/type/)
        """
        ...


    def getType(self) -> type["Object"]:
        """
        Get represented type (class)

        Returns
        - type (class) to be described.
        """
        ...


    def putListPropertyType(self, property: str, type: type["Object"]) -> None:
        """
        Specify that the property is a type-safe `List`.

        Arguments
        - property: name of the JavaBean property
        - type: class of List values
        """
        ...


    def putMapPropertyType(self, property: str, key: type["Object"], value: type["Object"]) -> None:
        """
        Specify that the property is a type-safe `Map`.

        Arguments
        - property: property name of this JavaBean
        - key: class of keys in Map
        - value: class of values in Map
        """
        ...


    def addPropertyParameters(self, pName: str, *classes: Tuple[type[Any], ...]) -> None:
        """
        Adds new substitute for property `pName` parameterized by `classes` to
        this `TypeDescription`. If `pName` has been added before - updates
        parameters with `classes`.

        Arguments
        - pName: - parameter name
        - classes: - parameterized by
        """
        ...


    def toString(self) -> str:
        ...


    def getProperty(self, name: str) -> "Property":
        ...


    def substituteProperty(self, pName: str, pType: type[Any], getter: str, setter: str, *argParams: Tuple[type[Any], ...]) -> None:
        """
        Adds property substitute for `pName`

        Arguments
        - pName: property name
        - pType: property type
        - getter: method name for getter
        - setter: method name for setter
        - argParams: actual types for parameterized type (List&lt;?&gt;, Map&lt;?&gt;)
        """
        ...


    def substituteProperty(self, substitute: "PropertySubstitute") -> None:
        ...


    def setPropertyUtils(self, propertyUtils: "PropertyUtils") -> None:
        ...


    def setIncludes(self, *propNames: Tuple[str, ...]) -> None:
        ...


    def setExcludes(self, *propNames: Tuple[str, ...]) -> None:
        ...


    def getProperties(self) -> set["Property"]:
        ...


    def setupPropertyType(self, key: str, valueNode: "Node") -> bool:
        ...


    def setProperty(self, targetBean: "Object", propertyName: str, value: "Object") -> bool:
        ...


    def newInstance(self, node: "Node") -> "Object":
        """
        This method should be overridden for TypeDescription implementations that are supposed to
        implement instantiation logic that is different from default one as implemented in YAML
        constructors. Note that even if you override this method, default filling of fields with
        variables from parsed YAML will still occur later.

        Arguments
        - node: - node to construct the instance from

        Returns
        - new instance
        """
        ...


    def newInstance(self, propertyName: str, node: "Node") -> "Object":
        ...


    def finalizeConstruction(self, obj: "Object") -> "Object":
        """
        Is invoked after entity is filled with values from deserialized YAML

        Arguments
        - obj: - deserialized entity

        Returns
        - postprocessed deserialized entity
        """
        ...



"""
Python module generated from Java source file org.bukkit.conversations.Prompt

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.conversations import *
from typing import Any, Callable, Iterable, Tuple


class Prompt(Cloneable):
    """
    A Prompt is the main constituent of a Conversation. Each prompt
    displays text to the user and optionally waits for a user's response.
    Prompts are chained together into a directed graph that represents the
    conversation flow. To halt a conversation, END_OF_CONVERSATION is returned
    in liu of another Prompt object.
    """

    END_OF_CONVERSATION = None
    """
    A convenience constant for indicating the end of a conversation.
    """


    def getPromptText(self, context: "ConversationContext") -> str:
        """
        Gets the text to display to the user when this prompt is first
        presented.

        Arguments
        - context: Context information about the conversation.

        Returns
        - The text to display.
        """
        ...


    def blocksForInput(self, context: "ConversationContext") -> bool:
        """
        Checks to see if this prompt implementation should wait for user input
        or immediately display the next prompt.

        Arguments
        - context: Context information about the conversation.

        Returns
        - If True, the Conversation will wait for input before
            continuing. If False, .acceptInput(ConversationContext, String) will be called immediately with `null` input.
        """
        ...


    def acceptInput(self, context: "ConversationContext", input: str) -> "Prompt":
        """
        Accepts and processes input from the user. Using the input, the next
        Prompt in the prompt graph is returned.

        Arguments
        - context: Context information about the conversation.
        - input: The input text from the user.

        Returns
        - The next Prompt in the prompt graph.
        """
        ...



"""
Python module generated from Java source file com.google.common.eventbus.Subscriber

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import VisibleForTesting
from com.google.common.eventbus import *
from com.google.j2objc.annotations import Weak
from java.lang.reflect import InvocationTargetException
from java.lang.reflect import Method
from java.util.concurrent import Executor
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Subscriber:
    """
    A subscriber method on a specific object, plus the executor that should be used for dispatching
    events to it.
    
    Two subscribers are equivalent when they refer to the same method on the same object (not
    class). This property is used to ensure that no subscriber method is registered more than once.

    Author(s)
    - Colin Decker
    """

    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockMultiPlaceEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import ImmutableList
from org.bukkit.block import Block
from org.bukkit.block import BlockState
from org.bukkit.entity import Player
from org.bukkit.event.block import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class BlockMultiPlaceEvent(BlockPlaceEvent):
    """
    Fired when a single block placement action of a player triggers the
    creation of multiple blocks(e.g. placing a bed block). The block returned
    by .getBlockPlaced() and its related methods is the block where
    the placed block would exist if the placement only affected a single
    block.
    """

    def __init__(self, states: list["BlockState"], clicked: "Block", itemInHand: "ItemStack", thePlayer: "Player", canBuild: bool):
        ...


    def getReplacedBlockStates(self) -> list["BlockState"]:
        """
        Gets a list of blockstates for all blocks which were replaced by the
        placement of the new blocks. Most of these blocks will just have a
        Material type of AIR.

        Returns
        - immutable list of replaced BlockStates
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.Lockable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class Lockable:
    """
    Represents a block (usually a container) that may be locked. When a lock is
    active an item with a name corresponding to the key will be required to open
    this block.
    """

    def isLocked(self) -> bool:
        """
        Checks if the container has a valid (non empty) key.

        Returns
        - True if the key is valid.
        """
        ...


    def getLock(self) -> str:
        """
        Gets the key needed to access the container.

        Returns
        - the key needed.

        Deprecated
        - locks are not necessarily pure strings
        """
        ...


    def setLock(self, key: str) -> None:
        """
        Sets the key required to access this container. Set to null (or empty
        string) to remove key.

        Arguments
        - key: the key required to access the container.

        Deprecated
        - locks are not necessarily pure strings
        """
        ...


    def setLockItem(self, key: "ItemStack") -> None:
        """
        Sets the key required to access this container. All explicit
        modifications to the set key will be required to match on the opening
        key. Set to null to remove key.

        Arguments
        - key: the key required to access the container.
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.Bisected

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.data import *
from typing import Any, Callable, Iterable, Tuple


class Bisected(BlockData):
    """
    'half' denotes which half of a two block tall material this block is.
    
    In game it may be referred to as either (top, bottom) or (upper, lower).
    """

    def getHalf(self) -> "Half":
        """
        Gets the value of the 'half' property.

        Returns
        - the 'half' value
        """
        ...


    def setHalf(self, half: "Half") -> None:
        """
        Sets the value of the 'half' property.

        Arguments
        - half: the new 'half' value
        """
        ...


    class Half(Enum):
        """
        The half of a vertically bisected block.
        """

        TOP = 0
        """
        The top half of the block, normally with the higher y coordinate.
        """
        BOTTOM = 1
        """
        The bottom half of the block, normally with the lower y coordinate.
        """



"""
Python module generated from Java source file com.google.common.util.concurrent.AggregateFuture

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import ImmutableCollection
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import ForOverride
from com.google.errorprone.annotations import OverridingMethodsMustInvokeSuper
from com.google.errorprone.annotations.concurrent import LazyInit
from java.util.concurrent import ExecutionException
from java.util.concurrent import Future
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AggregateFuture(AggregateFutureState):
    """
    A future whose value is derived from a collection of input futures.

    Arguments
    - <InputT>: the type of the individual inputs
    - <OutputT>: the type of the output (i.e. this) future
    """





from .Event import Event
from .EventHandler import EventHandler
from .Listener import Listener
from .HandlerList import HandlerList
from .Cancellable import Cancellable
from .EventPriority import EventPriority
from .EventException import EventException


"""
Python module generated from Java source file org.bukkit.inventory.FurnaceRecipe

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Collections
from org.bukkit import Material
from org.bukkit import NamespacedKey
from org.bukkit.inventory import *
from org.bukkit.material import MaterialData
from typing import Any, Callable, Iterable, Tuple


class FurnaceRecipe(CookingRecipe):
    """
    Represents a furnace recipe.
    """

    def __init__(self, result: "ItemStack", source: "Material"):
        ...


    def __init__(self, result: "ItemStack", source: "MaterialData"):
        ...


    def __init__(self, result: "ItemStack", source: "MaterialData", experience: float):
        ...


    def __init__(self, result: "ItemStack", source: "Material", data: int):
        ...


    def __init__(self, key: "NamespacedKey", result: "ItemStack", source: "Material", experience: float, cookingTime: int):
        """
        Create a furnace recipe to craft the specified ItemStack.

        Arguments
        - key: The unique recipe key
        - result: The item you want the recipe to create.
        - source: The input material.
        - experience: The experience given by this recipe
        - cookingTime: The cooking time (in ticks)
        """
        ...


    def __init__(self, key: "NamespacedKey", result: "ItemStack", source: "Material", data: int, experience: float, cookingTime: int):
        ...


    def __init__(self, key: "NamespacedKey", result: "ItemStack", input: "RecipeChoice", experience: float, cookingTime: int):
        """
        Create a furnace recipe to craft the specified ItemStack.

        Arguments
        - key: The unique recipe key
        - result: The item you want the recipe to create.
        - input: The input choices.
        - experience: The experience given by this recipe
        - cookingTime: The cooking time (in ticks)
        """
        ...


    def setInput(self, input: "MaterialData") -> "FurnaceRecipe":
        """
        Sets the input of this furnace recipe.

        Arguments
        - input: The input material.

        Returns
        - The changed recipe, so you can chain calls.
        """
        ...


    def setInput(self, input: "Material") -> "FurnaceRecipe":
        ...


    def setInput(self, input: "Material", data: int) -> "FurnaceRecipe":
        """
        Sets the input of this furnace recipe.

        Arguments
        - input: The input material.
        - data: The data value. (Note: This is currently ignored by the
            CraftBukkit server.)

        Returns
        - The changed recipe, so you can chain calls.

        Deprecated
        - Magic value
        """
        ...


    def setInputChoice(self, input: "RecipeChoice") -> "FurnaceRecipe":
        ...
